# /*##########################################################################
# Copyright (C) 2021 Timo Fuchs
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ############################################################################*/
"""Tests for fioh5"""
import numpy
import os
import tempfile
import unittest

from ..fioh5 import FioH5, is_fiofile, logger1, dtypeConverter


__authors__ = ["T. Fuchs"]
__license__ = "MIT"
__date__ = "15/10/2021"

fioftext = """
!
! Comments
!
%c
ascan omega 180.0 180.5 3:10/1 4
user username, acquisition started at Thu Dec 12 18:00:00 2021
sweep motor lag: 1.0e-03
channel 3: Detector
!
! Parameter
!
%p
channel3_exposure = 1.000000e+00
ScanName = ascan
!
! Data
!
%d
 Col 1 omega(encoder) DOUBLE
 Col 2 channel INTEGER
 Col 3 filename STRING
 Col 4 type STRING
 Col 5 unix time DOUBLE
 Col 6 enable BOOLEAN
 Col 7 time_s FLOAT
      179.998418821    3    00001    exposure    1576165741.20308    1    1.243
      180.048418821    3    00002    exposure    1576165742.20308    1    1.243
      180.098418821    3    00003    exposure    1576165743.20308    1    1.243
      180.148418821    3    00004    exposure    1576165744.20308    1    1.243
      180.198418821    3    00005    exposure    1576165745.20308    1    1.243
      180.248418821    3    00006    exposure    1576165746.20308    1    1.243
      180.298418821    3    00007    exposure    1576165747.20308    1    1.243
      180.348418821    3    00008    exposure    1576165748.20308    1    1.243
      180.398418821    3    00009    exposure    1576165749.20308    1    1.243
      180.448418821    3    00010    exposure    1576165750.20308    1    1.243
      nan    <no-data>    <no-data>    <no-data>    nan    <no-data>    <no-data>
"""


class TestFioH5(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.temp_dir = tempfile.TemporaryDirectory()
        # fd, cls.fname = tempfile.mkstemp()
        cls.fname_numbered = os.path.join(cls.temp_dir.name, "eh1scan_00005.fio")

        with open(cls.fname_numbered, "w") as fiof:
            fiof.write(fioftext)

    @classmethod
    def tearDownClass(cls):
        cls.temp_dir.cleanup()
        del cls.temp_dir

    def setUp(self):
        self.fioh5 = FioH5(self.fname_numbered)

    def tearDown(self):
        self.fioh5.close()

    def testScanNumber(self):
        # scan number is parsed from the file name.
        self.assertIn("/5.1", self.fioh5)
        self.assertIn("5.1", self.fioh5)

    def testContainsFile(self):
        self.assertIn("/5.1/measurement", self.fioh5)
        self.assertNotIn("25.2", self.fioh5)
        # measurement is a child of a scan, full path would be required to
        # access from root level
        self.assertNotIn("measurement", self.fioh5)
        # Groups may or may not have a trailing /
        self.assertIn("/5.1/measurement/", self.fioh5)
        self.assertIn("/5.1/measurement", self.fioh5)
        # Datasets can't have a trailing /
        self.assertIn("/5.1/measurement/omega(encoder)", self.fioh5)
        self.assertNotIn("/5.1/measurement/omega(encoder)/", self.fioh5)
        # No gamma
        self.assertNotIn("/5.1/measurement/gamma", self.fioh5)

    def testContainsGroup(self):
        self.assertIn("measurement", self.fioh5["/5.1/"])
        self.assertIn("measurement", self.fioh5["/5.1"])
        self.assertIn("5.1", self.fioh5["/"])
        self.assertNotIn("5.2", self.fioh5["/"])
        self.assertIn("measurement/filename", self.fioh5["/5.1"])
        # illegal trailing "/" after dataset name
        self.assertNotIn("measurement/filename/", self.fioh5["/5.1"])
        # full path to element in group (OK)
        self.assertIn("/5.1/measurement/filename", self.fioh5["/5.1/measurement"])

    def testDataType(self):
        meas = self.fioh5["/5.1/measurement/"]
        self.assertEqual(meas["omega(encoder)"].dtype, dtypeConverter["DOUBLE"])
        self.assertEqual(meas["channel"].dtype, dtypeConverter["INTEGER"])
        self.assertEqual(meas["filename"].dtype, dtypeConverter["STRING"])
        self.assertEqual(meas["time_s"].dtype, dtypeConverter["FLOAT"])
        self.assertEqual(meas["enable"].dtype, dtypeConverter["BOOLEAN"])

    def testDataColumn(self):
        self.assertAlmostEqual(
            sum(self.fioh5["/5.1/measurement/omega(encoder)"][:-1]), 1802.23418821
        )
        self.assertTrue(numpy.all(self.fioh5["/5.1/measurement/enable"][:-1]))

    # --- comment section tests ---

    def testComment(self):
        # should hold the complete comment section
        self.assertEqual(
            self.fioh5["/5.1/instrument/fiofile/comments"],
            """ascan omega 180.0 180.5 3:10/1 4
user username, acquisition started at Thu Dec 12 18:00:00 2021
sweep motor lag: 1.0e-03
channel 3: Detector
""",
        )

    def testDate(self):
        # there is no convention on how to format the time. So just check its existence.
        self.assertEqual(self.fioh5["/5.1/start_time"], "Thu Dec 12 18:00:00 2021")

    def testTitle(self):
        self.assertEqual(self.fioh5["/5.1/title"], "ascan omega 180.0 180.5 3:10/1 4")

    # --- parameter section tests ---

    def testParameter(self):
        # should hold the complete parameter section
        self.assertEqual(
            self.fioh5["/5.1/instrument/fiofile/parameter"],
            """channel3_exposure = 1.000000e+00
ScanName = ascan
""",
        )

    def testParsedParameter(self):
        # no dtype is given, so everything is str.
        self.assertEqual(
            self.fioh5["/5.1/instrument/parameter/channel3_exposure"], "1.000000e+00"
        )
        self.assertEqual(self.fioh5["/5.1/instrument/parameter/ScanName"], "ascan")

    def testNaNAndNoData(self):
        """Test access to nan and <no-data> fields from last row"""
        self.assertEqual(self.fioh5["/5.1/measurement/channel"][-1], -1)
        self.assertFalse(self.fioh5["/5.1/measurement/enable"][-1])
        self.assertTrue(numpy.isnan(self.fioh5["/5.1/measurement/time"][-1]))
        self.assertTrue(numpy.isnan(self.fioh5["/5.1/measurement/time_s"][-1]))

    def testNotFioH5(self):
        testfilename = os.path.join(self.temp_dir.name, "eh1scan_00010.fio")
        with open(testfilename, "w") as fiof:
            fiof.write("!Not a fio file!")

        self.assertRaises(IOError, FioH5, testfilename)

        self.assertTrue(is_fiofile(self.fname_numbered))
        self.assertFalse(is_fiofile(testfilename))

        os.unlink(testfilename)


class TestUnnumberedFioH5(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.temp_dir = tempfile.TemporaryDirectory()
        cls.fname_nosuffix = os.path.join(cls.temp_dir.name, "eh1scan_nosuffix.fio")

        with open(cls.fname_nosuffix, "w") as fiof:
            fiof.write(fioftext)

    @classmethod
    def tearDownClass(cls):
        cls.temp_dir.cleanup()
        del cls.temp_dir

    def setUp(self):
        self.fioh5 = FioH5(self.fname_nosuffix)

    def testLogMissingScanno(self):
        with self.assertLogs(logger1, level="WARNING") as cm:
            fioh5 = FioH5(self.fname_nosuffix)
        self.assertIn("Cannot parse scan number of file", cm.output[0])

    def testFallbackName(self):
        self.assertIn("/eh1scan_nosuffix", self.fioh5)


brokenHeaderText = """
!
! Comments
!
%c
ascan omega 180.0 180.5 3:10/1 4
user username, acquisited at Thu Dec 12 100 2021
sweep motor lavgvf.0e-03
channel 3: Detector
!
! Parameter
!
%p
channel3_exposu65 1.000000e+00
ScanName = ascan
!
! Data
!
%d
 Col 1 omega(encoder) DOUBLE
 Col 2 channel INTEGER
 Col 3 filename STRING
 Col 4 type STRING
 Col 5 unix time DOUBLE
      179.998418821    3    00001    exposure    1576165741.20308
      180.048418821    3    00002    exposure    1576165742.20308
      180.098418821    3    00003    exposure    1576165743.20308
      180.148418821    3    00004    exposure    1576165744.20308
      180.198418821    3    00005    exposure    1576165745.20308
      180.248418821    3    00006    exposure    1576165746.20308
      180.298418821    3    00007    exposure    1576165747.20308
      180.348418821    3    00008    exposure    1576165748.20308
      180.398418821    3    00009    exposure    1576165749.20308
      180.448418821    3    00010    exposure    1576165750.20308
"""


class TestBrokenHeaderFioH5(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.temp_dir = tempfile.TemporaryDirectory()
        cls.fname_numbered = os.path.join(cls.temp_dir.name, "eh1scan_00005.fio")

        with open(cls.fname_numbered, "w") as fiof:
            fiof.write(brokenHeaderText)

    @classmethod
    def tearDownClass(cls):
        cls.temp_dir.cleanup()
        del cls.temp_dir

    def setUp(self):
        self.fioh5 = FioH5(self.fname_numbered)

    def testLogBrokenHeader(self):
        with self.assertLogs(logger1, level="WARNING") as cm:
            fioh5 = FioH5(self.fname_numbered)
        self.assertIn("Cannot parse parameter section", cm.output[0])
        self.assertIn("Cannot parse default comment section", cm.output[1])

    def testComment(self):
        # should hold the complete comment section
        self.assertEqual(
            self.fioh5["/5.1/instrument/fiofile/comments"],
            """ascan omega 180.0 180.5 3:10/1 4
user username, acquisited at Thu Dec 12 100 2021
sweep motor lavgvf.0e-03
channel 3: Detector
""",
        )

    def testParameter(self):
        # should hold the complete parameter section
        self.assertEqual(
            self.fioh5["/5.1/instrument/fiofile/parameter"],
            """channel3_exposu65 1.000000e+00
ScanName = ascan
""",
        )



# /*##########################################################################
#
# Copyright (c) 2018-2019 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""This module provides tool bar helper.
"""

__authors__ = ["V. Valls"]
__license__ = "MIT"
__date__ = "28/06/2018"


import logging
import weakref
from typing import List, Optional

from silx.gui import qt
from silx.gui import icons
from .core import AlignmentMode
from .core import VisualizationMode
from .core import sift


_logger = logging.getLogger(__name__)


class AlignmentModeToolButton(qt.QToolButton):
    """ToolButton to select a AlignmentMode"""

    sigSelected = qt.Signal(AlignmentMode)

    def __init__(self, parent=None):
        super(AlignmentModeToolButton, self).__init__(parent=parent)

        menu = qt.QMenu(self)
        self.setMenu(menu)

        self.__group = qt.QActionGroup(self)
        self.__group.setExclusive(True)
        self.__group.triggered.connect(self.__selectionChanged)

        icon = icons.getQIcon("compare-align-origin")
        action = qt.QAction(icon, "Align images on their upper-left pixel", self)
        action.setProperty("enum", AlignmentMode.ORIGIN)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        self.__originAlignAction = action
        menu.addAction(action)
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-align-center")
        action = qt.QAction(icon, "Center images", self)
        action.setProperty("enum", AlignmentMode.CENTER)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        self.__centerAlignAction = action
        menu.addAction(action)
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-align-stretch")
        action = qt.QAction(icon, "Stretch the second image on the first one", self)
        action.setProperty("enum", AlignmentMode.STRETCH)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        self.__stretchAlignAction = action
        menu.addAction(action)
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-align-auto")
        action = qt.QAction(icon, "Auto-alignment of the second image", self)
        action.setProperty("enum", AlignmentMode.AUTO)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        self.__autoAlignAction = action
        menu.addAction(action)
        if sift is None:
            action.setEnabled(False)
            action.setToolTip("Sift module is not available")
        self.__group.addAction(action)

    def getActionFromMode(self, mode: AlignmentMode) -> Optional[qt.QAction]:
        """Returns an action from it's mode"""
        for action in self.__group.actions():
            actionMode = action.property("enum")
            if mode == actionMode:
                return action
        return None

    def setVisibleModes(self, modes: List[AlignmentMode]):
        """Make visible only a set of modes.

        The order does not matter.
        """
        modes = set(modes)
        for action in self.__group.actions():
            mode = action.property("enum")
            action.setVisible(mode in modes)

    def __selectionChanged(self, selectedAction: qt.QAction):
        """Called when user requesting changes of the alignment mode."""
        self.__updateMenu()
        mode = self.getSelected()
        self.sigSelected.emit(mode)

    def __updateMenu(self):
        """Update the state of the action containing alignment menu."""
        selectedAction = self.__group.checkedAction()
        if selectedAction is not None:
            self.setText(selectedAction.text())
            self.setIcon(selectedAction.icon())
            self.setToolTip(selectedAction.toolTip())
        else:
            self.setText("")
            self.setIcon(qt.QIcon())
            self.setToolTip("")

    def getSelected(self) -> AlignmentMode:
        action = self.__group.checkedAction()
        if action is None:
            return None
        return action.property("enum")

    def setSelected(self, mode: AlignmentMode):
        action = self.getActionFromMode(mode)
        old = self.__group.blockSignals(True)
        if action is not None:
            # Check this action
            action.setChecked(True)
        else:
            action = self.__group.checkedAction()
            if action is not None:
                # Uncheck this action
                action.setChecked(False)
        self.__updateMenu()
        self.__group.blockSignals(old)


class VisualizationModeToolButton(qt.QToolButton):
    """ToolButton to select a VisualisationMode"""

    sigSelected = qt.Signal(VisualizationMode)

    def __init__(self, parent=None):
        super(VisualizationModeToolButton, self).__init__(parent=parent)

        menu = qt.QMenu(self)
        self.setMenu(menu)

        self.__group = qt.QActionGroup(self)
        self.__group.setExclusive(True)
        self.__group.triggered.connect(self.__selectionChanged)

        icon = icons.getQIcon("compare-mode-a")
        action = qt.QAction(icon, "Display the first image only", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_A))
        action.setProperty("enum", VisualizationMode.ONLY_A)
        menu.addAction(action)
        self.__aModeAction = action
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-mode-b")
        action = qt.QAction(icon, "Display the second image only", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_B))
        action.setProperty("enum", VisualizationMode.ONLY_B)
        menu.addAction(action)
        self.__bModeAction = action
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-mode-vline")
        action = qt.QAction(icon, "Vertical compare mode", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_V))
        action.setProperty("enum", VisualizationMode.VERTICAL_LINE)
        menu.addAction(action)
        self.__vlineModeAction = action
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-mode-hline")
        action = qt.QAction(icon, "Horizontal compare mode", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_H))
        action.setProperty("enum", VisualizationMode.HORIZONTAL_LINE)
        menu.addAction(action)
        self.__hlineModeAction = action
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-mode-rb-channel")
        action = qt.QAction(icon, "Blue/red compare mode (additive mode)", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_C))
        action.setProperty("enum", VisualizationMode.COMPOSITE_RED_BLUE_GRAY)
        menu.addAction(action)
        self.__brChannelModeAction = action
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-mode-rbneg-channel")
        action = qt.QAction(icon, "Yellow/cyan compare mode (subtractive mode)", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_Y))
        action.setProperty("enum", VisualizationMode.COMPOSITE_RED_BLUE_GRAY_NEG)
        menu.addAction(action)
        self.__ycChannelModeAction = action
        self.__group.addAction(action)

        icon = icons.getQIcon("compare-mode-a-minus-b")
        action = qt.QAction(icon, "Raw A minus B compare mode", self)
        action.setIconVisibleInMenu(True)
        action.setCheckable(True)
        action.setShortcut(qt.QKeySequence(qt.Qt.Key_W))
        action.setProperty("enum", VisualizationMode.COMPOSITE_A_MINUS_B)
        menu.addAction(action)
        self.__ycChannelModeAction = action
        self.__group.addAction(action)

    def getActionFromMode(self, mode: VisualizationMode) -> Optional[qt.QAction]:
        """Returns an action from it's mode"""
        for action in self.__group.actions():
            actionMode = action.property("enum")
            if mode == actionMode:
                return action
        return None

    def setVisibleModes(self, modes: List[VisualizationMode]):
        """Make visible only a set of modes.

        The order does not matter.
        """
        modes = set(modes)
        for action in self.__group.actions():
            mode = action.property("enum")
            action.setVisible(mode in modes)

    def __selectionChanged(self, selectedAction: qt.QAction):
        """Called when user requesting changes of the visualization mode."""
        self.__updateMenu()
        mode = self.getSelected()
        self.sigSelected.emit(mode)

    def __updateMenu(self):
        """Update the state of the action containing visualization menu."""
        selectedAction = self.__group.checkedAction()
        if selectedAction is not None:
            self.setText(selectedAction.text())
            self.setIcon(selectedAction.icon())
            self.setToolTip(selectedAction.toolTip())
        else:
            self.setText("")
            self.setIcon(qt.QIcon())
            self.setToolTip("")

    def getSelected(self) -> VisualizationMode:
        action = self.__group.checkedAction()
        if action is None:
            return None
        return action.property("enum")

    def setSelected(self, mode: VisualizationMode):
        action = self.getActionFromMode(mode)
        old = self.__group.blockSignals(True)
        if action is not None:
            # Check this action
            action.setChecked(True)
        else:
            action = self.__group.checkedAction()
            if action is not None:
                # Uncheck this action
                action.setChecked(False)
        self.__updateMenu()
        self.__group.blockSignals(old)


class CompareImagesToolBar(qt.QToolBar):
    """ToolBar containing specific tools to custom the configuration of a
    :class:`CompareImages` widget

    Use :meth:`setCompareWidget` to connect this toolbar to a specific
    :class:`CompareImages` widget.

    :param Union[qt.QWidget,None] parent: Parent of this widget.
    """

    def __init__(self, parent=None):
        qt.QToolBar.__init__(self, parent)
        self.setWindowTitle("Compare images")

        self.__compareWidget = None

        self.__visualizationToolButton = VisualizationModeToolButton(self)
        self.__visualizationToolButton.setPopupMode(qt.QToolButton.InstantPopup)
        self.__visualizationToolButton.sigSelected.connect(self.__visualizationChanged)
        self.addWidget(self.__visualizationToolButton)

        self.__alignmentToolButton = AlignmentModeToolButton(self)
        self.__alignmentToolButton.setPopupMode(qt.QToolButton.InstantPopup)
        self.__alignmentToolButton.sigSelected.connect(self.__alignmentChanged)
        self.addWidget(self.__alignmentToolButton)

        icon = icons.getQIcon("compare-keypoints")
        action = qt.QAction(icon, "Display/hide alignment keypoints", self)
        action.setCheckable(True)
        action.triggered.connect(self.__keypointVisibilityChanged)
        self.addAction(action)
        self.__displayKeypoints = action

    def __visualizationChanged(self, mode: VisualizationMode):
        widget = self.getCompareWidget()
        if widget is not None:
            widget.setVisualizationMode(mode)

    def __alignmentChanged(self, mode: AlignmentMode):
        widget = self.getCompareWidget()
        if widget is not None:
            widget.setAlignmentMode(mode)

    def setCompareWidget(self, widget):
        """
        Connect this tool bar to a specific :class:`CompareImages` widget.

        :param Union[None,CompareImages] widget: The widget to connect with.
        """
        compareWidget = self.getCompareWidget()
        if compareWidget is not None:
            compareWidget.sigConfigurationChanged.disconnect(
                self.__updateSelectedActions
            )
        compareWidget = widget
        self.setEnabled(compareWidget is not None)
        if compareWidget is None:
            self.__compareWidget = None
        else:
            self.__compareWidget = weakref.ref(compareWidget)
        if compareWidget is not None:
            widget.sigConfigurationChanged.connect(self.__updateSelectedActions)
        self.__updateSelectedActions()

    def getCompareWidget(self):
        """Returns the connected widget.

        :rtype: CompareImages
        """
        if self.__compareWidget is None:
            return None
        else:
            return self.__compareWidget()

    def __updateSelectedActions(self):
        """
        Update the state of this tool bar according to the state of the
        connected :class:`CompareImages` widget.
        """
        widget = self.getCompareWidget()
        if widget is None:
            return
        self.__visualizationToolButton.setSelected(widget.getVisualizationMode())
        self.__alignmentToolButton.setSelected(widget.getAlignmentMode())
        self.__displayKeypoints.setChecked(widget.getKeypointsVisible())

    def __keypointVisibilityChanged(self):
        """Called when action managing keypoints visibility changes"""
        widget = self.getCompareWidget()
        if widget is not None:
            keypointsVisible = self.__displayKeypoints.isChecked()
            widget.setKeypointsVisible(keypointsVisible)



# /*##########################################################################
#
# Copyright (c) 2017-2021 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/

"""
This script compares the rendering of PlotWidget's matplotlib and OpenGL backends.
"""

from __future__ import annotations

__license__ = "MIT"

import numpy
import sys
import functools

from silx.gui import qt

from silx.gui.plot import PlotWidget
from silx.gui.plot import items
from silx.gui.plot.items.marker import Marker
from silx.gui.plot.utils.axis import SyncAxes


_DESCRIPTIONS = {}


class MyPlotWindow(qt.QMainWindow):
    """QMainWindow with selected tools"""

    def __init__(self, parent=None):
        super(MyPlotWindow, self).__init__(parent)

        # Create a PlotWidget
        self._plot1 = PlotWidget(parent=self, backend="mpl")
        self._plot1.setGraphTitle("matplotlib")
        self._plot2 = PlotWidget(parent=self, backend="opengl")
        self._plot2.setGraphTitle("opengl")

        self.constraintX = SyncAxes(
            [
                self._plot1.getXAxis(),
                self._plot2.getXAxis(),
            ]
        )
        self.constraintY = SyncAxes(
            [
                self._plot1.getYAxis(),
                self._plot2.getYAxis(),
            ]
        )

        plotWidget = qt.QWidget(self)
        plotLayout = qt.QHBoxLayout(plotWidget)
        plotLayout.addWidget(self._plot1)
        plotLayout.addWidget(self._plot2)
        plotLayout.setContentsMargins(0, 0, 0, 0)
        plotLayout.setContentsMargins(0, 0, 0, 0)

        options = self.createOptions(self)
        centralWidget = qt.QWidget(self)
        layout = qt.QHBoxLayout(centralWidget)
        layout.setSpacing(0)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(options)
        layout.addWidget(plotWidget)

        self.setCentralWidget(centralWidget)

        self._state = {}

    def clear(self):
        self._state = {}

    def createOptions(self, parent):
        options = qt.QWidget(parent)
        layout = qt.QVBoxLayout(options)
        for id, description in _DESCRIPTIONS.items():
            label, _func = description
            button = qt.QPushButton(label, self)
            button.clicked.connect(functools.partial(self.showUseCase, id))
            layout.addWidget(button)
        layout.addStretch()
        return options

    def showUseCase(self, name: str):
        description = _DESCRIPTIONS.get(name)
        if description is None:
            raise ValueError(f"Unknown use case '{name}'")
        setupFunc = description[1]
        self.clear()
        for p in [self._plot1, self._plot2]:
            p.clear()
            setupFunc(self, p)
            p.resetZoom()

    def _register(name, label):
        def decorator(func):
            _DESCRIPTIONS[name] = (label, func)
            return func

        return decorator

    def _addLine(
        self,
        plot,
        lineWidth: float,
        lineStyle: str,
        color: str,
        gapColor: str | None,
        curve: bool,
    ):
        state = self._state.setdefault(plot, {})
        x = state.get("x", 0)
        y = state.get("y", 0)
        x += 10
        state["x"] = x
        state["y"] = y

        start = (x - 20, y + 0)
        stop = (x + 40, y + 100)

        def createShape():
            shape = items.Shape("polylines")
            shape.setPoints(numpy.array((start, stop)))
            shape.setLineWidth(lineWidth)
            shape.setLineStyle(lineStyle)
            shape.setColor(color)
            if gapColor is not None:
                shape.setLineGapColor(gapColor)
            return shape

        def createCurve():
            curve = items.Curve()
            array = numpy.array((start, stop)).T
            curve.setData(array[0], array[1])
            curve.setLineWidth(lineWidth)
            curve.setLineStyle(lineStyle)
            curve.setColor(color)
            curve.setSymbol("")
            if gapColor is not None:
                curve.setLineGapColor(gapColor)
            return curve

        if curve:
            plot.addItem(createCurve())
        else:
            plot.addItem(createShape())

    @_register("linewidth", "Line width")
    def _setupLineStyle(self, plot: PlotWidget):
        self._addLine(plot, 0.5, "-", "#0000FF", None, curve=False)
        self._addLine(plot, 1.0, "-", "#0000FF", None, curve=False)
        self._addLine(plot, 2.0, "-", "#0000FF", None, curve=False)
        self._addLine(plot, 4.0, "-", "#0000FF", None, curve=False)
        self._addLine(plot, 0.5, "-", "#00FFFF", None, curve=True)
        self._addLine(plot, 1.0, "-", "#00FFFF", None, curve=True)
        self._addLine(plot, 2.0, "-", "#00FFFF", None, curve=True)
        self._addLine(plot, 4.0, "-", "#00FFFF", None, curve=True)

    @_register("linestyle", "Line style")
    def _setupLineStyle(self, plot: PlotWidget):
        self._addLine(plot, 1.0, "--", "#0000FF", None, curve=False)
        self._addLine(plot, 1.0, "-.", "#0000FF", None, curve=False)
        self._addLine(plot, 1.0, ":", "#0000FF", None, curve=False)
        self._addLine(plot, 2.0, "--", "#00FFFF", None, curve=True)
        self._addLine(plot, 2.0, "-.", "#00FFFF", None, curve=True)
        self._addLine(plot, 2.0, ":", "#00FFFF", None, curve=True)

    @_register("gapcolor", "LineStyle Gap Color")
    def _setupLineStyleGapColor(self, plot):
        self._addLine(plot, 1.0, "-", "#FF00FF", "black", curve=False)
        self._addLine(plot, 1.0, "-.", "#FF00FF", "black", curve=False)
        self._addLine(plot, 1.0, "--", "#FF00FF", "black", curve=False)
        self._addLine(plot, 0.5, "--", "#FF00FF", "black", curve=False)
        self._addLine(plot, 1.5, "--", "#FF00FF", "black", curve=False)
        self._addLine(plot, 2.0, "--", "#FF00FF", "black", curve=False)
        plot.setGraphXLimits(0, 100)
        plot.setGraphYLimits(0, 100)

    @_register("curveshape", "Curve vs Shape")
    def _setupLineStyleCurveShape(self, plot):
        self._addLine(plot, 1.0, (0, (5, 5)), "#00FF00", None, curve=False)
        self._addLine(plot, 4.0, (0, (3, 3)), "#00FF00", None, curve=False)
        self._addLine(plot, 4.0, (0, (5, 5)), "#00FF00", None, curve=False)
        self._addLine(plot, 4.0, (0, (7, 7)), "#00FF00", None, curve=False)
        self._addLine(plot, 1.0, (0, (5, 5)), "#00FFFF", None, curve=True)
        self._addLine(plot, 4.0, (0, (3, 3)), "#00FFFF", None, curve=True)
        self._addLine(plot, 4.0, (0, (5, 5)), "#00FFFF", None, curve=True)
        self._addLine(plot, 4.0, (0, (7, 7)), "#00FFFF", None, curve=True)
        plot.setGraphXLimits(0, 100)
        plot.setGraphYLimits(0, 100)

    @_register("text", "Text")
    def _setupText(self, plot):
        plot.getDefaultColormap().setName("viridis")

        # Add an image to the plot
        x = numpy.outer(numpy.linspace(-10, 10, 200), numpy.linspace(-10, 5, 150))
        image = numpy.sin(x) / x
        plot.addImage(image)

        label = Marker()
        label.setPosition(40, 150)
        label.setText("No background")
        plot.addItem(label)

        label = Marker()
        label.setPosition(50, 50)
        label.setText("Foo bar\nmmmmmmmmmmmmmmmmmmmm")
        label.setBackgroundColor("#FFFFFF44")
        plot.addItem(label)

        label2 = Marker()
        label2.setPosition(70, 70)
        label2.setText("Foo bar")
        label2.setColor("red")
        label2.setBackgroundColor("#00000044")
        plot.addItem(label2)

        label3 = Marker()
        label3.setPosition(10, 70)
        label3.setText("Pioupiou")
        label3.setColor("yellow")
        label3.setBackgroundColor("#000000")
        plot.addItem(label3)

    @_register("marker", "Marker")
    def _setupMarker(self, plot):
        plot.getDefaultColormap().setName("viridis")

        # Add an image to the plot
        x = numpy.outer(numpy.linspace(-10, 10, 200), numpy.linspace(-10, 5, 150))
        image = numpy.sin(x) / x
        plot.addImage(image)

        label = Marker()
        label.setSymbol("o")
        label.setPosition(30, 30)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol(".")
        label.setPosition(50, 30)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol(",")
        label.setPosition(70, 30)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol("+")
        # label.setSymbolSize(100)
        label.setPosition(30, 50)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol("x")
        label.setPosition(50, 50)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol("d")
        label.setPosition(70, 50)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol("s")
        label.setPosition(30, 70)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol("|")
        label.setPosition(50, 70)
        label.setColor("white")
        plot.addItem(label)

        label = Marker()
        label.setSymbol("_")
        label.setPosition(70, 70)
        label.setColor("white")
        plot.addItem(label)

    @_register("arrows", "Arrows")
    def _setupArrows(self, plot):
        """Display few lines with markers."""
        plot.setDataMargins(0.1, 0.1, 0.1, 0.1)

        plot.addCurve(
            x=[-10, 0, 0, -10, -10], y=[90, 90, 10, 10, 90], legend="box1", color="gray"
        )
        plot.addCurve(
            x=[110, 100, 100, 110, 110],
            y=[90, 90, 10, 10, 90],
            legend="box2",
            color="gray",
        )
        plot.addCurve(
            y=[-10, 0, 0, -10, -10], x=[90, 90, 10, 10, 90], legend="box3", color="gray"
        )
        plot.addCurve(
            y=[110, 100, 100, 110, 110],
            x=[90, 90, 10, 10, 90],
            legend="box4",
            color="gray",
        )

        def addCompositeLine(
            source, destination, symbolSource, symbolDestination, legend, color
        ):
            line = numpy.array([source, destination]).T
            plot.addCurve(x=line[0, :], y=line[1, :], color=color, legend=legend)
            plot.addMarker(x=source[0], y=source[1], symbol=symbolSource, color=color)
            plot.addMarker(
                x=destination[0],
                y=destination[1],
                symbol=symbolDestination,
                color=color,
            )

        addCompositeLine([0, 50], [100, 50], "caretleft", "caretright", "l1", "red")
        addCompositeLine([0, 30], [100, 30], "tickup", "tickdown", "l2", "blue")
        addCompositeLine([0, 70], [100, 70], "|", "|", "l3", "black")

        addCompositeLine([50, 0], [50, 100], "caretdown", "caretup", "l4", "red")
        addCompositeLine([30, 0], [30, 100], "tickleft", "tickright", "l5", "blue")
        addCompositeLine([70, 0], [70, 100], "_", "_", "l6", "black")


def main():
    global app
    app = qt.QApplication([])

    # Create the ad hoc window containing a PlotWidget and associated tools
    window = MyPlotWindow()
    window.setAttribute(qt.Qt.WA_DeleteOnClose)
    window.show()
    if len(sys.argv) == 1:
        useCase = "linestyle"
    else:
        useCase = sys.argv[1]
    window.showUseCase(useCase)
    app.exec()


if __name__ == "__main__":
    main()



# /*##########################################################################
#
# Copyright (c) 2004-2023 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# ###########################################################################*/

"""Custom plot selection window for selecting 1D datasets to plot."""
from __future__ import annotations

from silx.gui import qt, plot, icons
import silx.gui
import silx.gui.plot
import silx.gui.plot.items
import silx.io
import numpy
import functools
from silx.gui.plot.LegendSelector import LegendIcon
import silx.io.url
import silx.io.utils
from silx.gui.hdf5 import _utils
import weakref


# Custom role for highlighting the drop zones
_DROP_HIGHLIGHT_ROLE = qt.Qt.UserRole + 1


class _HashDropZones(qt.QStyledItemDelegate):
    """Delegate item displaying a drop zone when the item does not contain a dataset."""

    def __init__(self, parent: qt.QWidget | None = None):
        super(_HashDropZones, self).__init__(parent)
        self.__dropPen = qt.QPen(qt.QColor("#D0D0D0"), 2, qt.Qt.DotLine)
        self.__highlightDropPen = qt.QPen(qt.QColor("#000000"), 2, qt.Qt.SolidLine)
        self.__dropTargetIndex = None

    def paint(self, painter, option, index):
        """Paint the item"""
        displayDropZone = False
        isDropHighlighted = False
        if index.isValid():
            model = index.model()
            item = model.itemFromIndex(index)
            isDropHighlighted = item.data(_DROP_HIGHLIGHT_ROLE)

            rowIndex = model.index(index.row(), 1, index.parent())
            rowItem = model.itemFromIndex(rowIndex)
            if not rowItem.data(qt.Qt.DisplayRole):
                parentIndex = model.index(
                    index.parent().row(), 1, index.parent().parent()
                )
                parentItem = model.itemFromIndex(parentIndex)
                if parentItem and parentItem.text() not in ["Y"] or index.row() == 0:
                    displayDropZone = True

        if isDropHighlighted:
            painter.save()
            painter.setPen(self.__highlightDropPen)
            painter.drawRect(option.rect.adjusted(1, 1, -1, -1))
            painter.restore()

        if displayDropZone:
            painter.save()

            # Draw background if selected
            if option.state & qt.QStyle.State_Selected:
                colorGroup = qt.QPalette.Inactive
                if option.state & qt.QStyle.State_Active:
                    colorGroup = qt.QPalette.Active
                if not option.state & qt.QStyle.State_Enabled:
                    colorGroup = qt.QPalette.Disabled
                brush = option.palette.brush(colorGroup, qt.QPalette.Highlight)
                painter.fillRect(option.rect, brush)

            isDropTarget = self.__dropTargetIndex == index
            painter.setPen(self.__highlightDropPen if isDropTarget else self.__dropPen)
            painter.drawRect(option.rect.adjusted(3, 3, -3, -3))

            painter.drawText(
                option.rect.adjusted(3, 3, -3, -3),
                qt.Qt.AlignLeft | qt.Qt.AlignVCenter,
                " Drop a 1D dataset",
            )
            painter.restore()
        else:
            qt.QStyledItemDelegate.paint(self, painter, option, index)


class _FileListModel(qt.QStandardItemModel):
    """Model for displaying dropped file names in a TreeView widget"""

    def __init__(self, plot, parent=None):
        super().__init__(parent)
        root = self.invisibleRootItem()
        root.setDragEnabled(False)

        self.setColumnCount(3)
        self.setHorizontalHeaderLabels(["", "Dataset", ""])

        self._xParent = qt.QStandardItem("X")
        self._xParent.setEditable(False)
        root.appendRow(self._xParent)

        self._yParent = qt.QStandardItem("Y")
        self._yParent.setEditable(False)
        root.appendRow(self._yParent)

        self._plot1D = weakref.proxy(plot)
        self._index = 0

        self._xDataset = None

        self._reset()

        self.addUrl(silx.io.url.DataUrl())

        self.rowsAboutToBeRemoved.connect(self._rowsAboutToBeRemoved)

    def getXParent(self) -> qt.QStandardItem:
        """Return the parent item for the X dataset"""
        return self._xParent

    def getYParent(self) -> qt.QStandardItem:
        """Return the parent item for the Y datasets"""
        return self._yParent

    def _setXFile(self, url: silx.io.url.DataUrl | None = None):
        """Set the X dataset file in the model"""
        if url is None:
            dataName = ""
        else:
            dataName = self._getBasename(url.data_path())
            self._plot1D.setXAxisLabel(dataName)

        fileItem, iconItem, removeItem = self._createRowItems(dataName, None)
        fileItem.setData(qt.QSize(0, 30), qt.Qt.SizeHintRole)

        fileItem.setData(self._createToolTip(url), qt.Qt.ToolTipRole)

        xIndex = self._findRowContainingText("X")
        if xIndex is not None:
            self.setItem(xIndex, 1, fileItem)
            self.setItem(xIndex, 2, removeItem)

    def _addYFile(
        self,
        url: silx.io.url.DataUrl | None = None,
        curve: silx.gui.plot.items.Curve | None = None,
    ):
        """Add a Y dataset file to the model"""
        if url is None:
            dataName = ""
        else:
            dataName = self._getBasename(url.data_path())
        fileItem, iconItem, removeItem = self._createRowItems(dataName, curve)
        if not dataName:
            # provide size hint for 'empty' item
            fileItem.setData(qt.QSize(0, 30), qt.Qt.SizeHintRole)

        fileItem.setData(self._createToolTip(url), qt.Qt.ToolTipRole)

        if self.getYParent().rowCount() == 0:
            self.getYParent().appendRow([qt.QStandardItem(), fileItem])
            return

        self.getYParent().insertRow(
            self.getYParent().rowCount() - 1, [iconItem, fileItem, removeItem]
        )

    def _findRowContainingText(self, text: str) -> int | None:
        """Return the row index containing the given text, or None if not found."""
        for row in range(self.rowCount()):
            item = self.item(row, 0)
            if item and item.text() == text:
                return row
        return None

    def _createRowItems(
        self, filename: str, curve: silx.gui.plot.items.Curve | None
    ) -> tuple[qt.QStandardItem, qt.QStandardItem, qt.QStandardItem]:
        """Create the items for a row in the model"""
        fileItem = qt.QStandardItem(filename)
        fileItem.setData(curve, qt.Qt.UserRole)
        fileItem.setData(False, _DROP_HIGHLIGHT_ROLE)
        fileItem.setEditable(False)
        fileItem.setDropEnabled(True)

        iconItem = qt.QStandardItem()
        removeItem = qt.QStandardItem()
        iconItem.setEditable(False)

        return fileItem, iconItem, removeItem

    def addUrl(self, url: silx.io.url.DataUrl, node: str = "X"):
        """Add a dataset to the model"""
        if url.file_path() is not None:
            file = silx.io.open(url.file_path())
            data = file[url.data_path()]
            if silx.io.is_dataset(data) and data.ndim == 1:
                if node == "X":
                    self._xDataset = data

                    for item in self._plot1D.getItems():
                        y = item.getInfo()
                        if y is None:
                            continue
                        length = min(len(self._xDataset), len(y))
                        item.setData(self._xDataset[:length], y[:length])

                    self._setXFile(url)
                else:
                    if data is None:
                        return
                    curve = self._addPlot(self._xDataset, data)
                    self._addYFile(url, curve)

    def _addPlot(
        self, x: numpy.ndarray | None, y: numpy.ndarray
    ) -> silx.gui.plot.items.Curve:
        """Add a curve to the plot with the given x and y data."""
        if x is None:
            x = numpy.arange(len(y))

        legend = f"Curve {self._index}"
        self._index += 1

        length = min(len(x), len(y))
        curve = self._plot1D.addCurve(x=x[:length], y=y[:length], legend=legend)
        curve.setInfo(y[()])
        return curve

    def _rowsAboutToBeRemoved(self, parentIndex: qt.QModelIndex, first: int, last: int):
        """Remove the curves from the plot when a row is removed from the model."""
        parentItem = self.itemFromIndex(parentIndex)
        for row in range(first, last + 1):
            fileItem = parentItem.child(row, 1)
            curve = fileItem.data(qt.Qt.UserRole)
            if curve is not None:
                self._plot1D.removeItem(curve)

    def _updateYCurvesWithDefaultX(self):
        """Update the Y curves with the default X dataset."""
        for item in self._plot1D.getItems():
            y = item.getInfo()
            x = numpy.arange(len(y))
            item.setData(x, y)
            self._plot1D.resetZoom()

    def clearAll(self):
        """Clear all datasets from the model and the plot."""
        self._xDataset = None

        self._yParent.removeRows(0, self._yParent.rowCount())

        self._setXFile()
        self._addYFile()

        self._plot1D.clear()
        self._plot1D.setXAxisLabel("X")
        self._plot1D.resetZoom()

        self._updateYCurvesWithDefaultX()

    def _reset(self):
        """Reset the model to its initial state."""
        self._xDataset = None

        self._setXFile()
        self._addYFile()

        self._plot1D.clear()
        self._updateYCurvesWithDefaultX()

    def _createToolTip(self, url: silx.io.url.DataUrl) -> str:
        """Create the tooltip for a dataset."""
        if url is None:
            return ""
        attrs = {
            "Name": self._getBasename(url.data_path()),
            "Path": url.data_path(),
            "File name": self._getBasename(url.file_path()),
        }
        return _utils.htmlFromDict(attrs, title="HDF5 Dataset")

    def _getBasename(self, text: str) -> str:
        """Return the basename of a file path."""
        return text.split("/")[-1]


class _DropTreeView(qt.QTreeView):
    """TreeView widget for displaying dropped file names"""

    (_DESCRIPTION_COLUMN, _FILE_COLUMN, _REMOVE_COLUMN) = range(3)

    def __init__(self, model, parent=None):
        super().__init__(parent)
        self.setModel(model)

        self.setItemDelegateForColumn(1, _HashDropZones(self))

        header = self.header()
        header.setStretchLastSection(False)
        header.setSectionResizeMode(0, qt.QHeaderView.ResizeToContents)
        header.setSectionResizeMode(1, qt.QHeaderView.Stretch)
        header.setSectionResizeMode(2, qt.QHeaderView.ResizeToContents)
        self.expandAll()

        self.setSelectionMode(qt.QAbstractItemView.SingleSelection)
        self.setItemsExpandable(False)
        self.setSelectionMode(qt.QAbstractItemView.NoSelection)
        self.setFocusPolicy(qt.Qt.NoFocus)
        self.setDropIndicatorShown(True)
        self.setDragDropOverwriteMode(True)
        self.setDragEnabled(False)
        self.viewport().setAcceptDrops(True)

    def _createIconWidget(self, row: int, parentItem: qt.QStandardItem):
        """Create the icon widget for a row in the model"""
        fileItem = parentItem.child(row, 1)
        if fileItem is None:
            return
        iconItem = parentItem.child(row, 0)
        curve = fileItem.data(qt.Qt.UserRole)
        if curve is None:
            return

        legendIcon = LegendIcon(None, curve)
        widget = qt.QWidget()
        layout = qt.QHBoxLayout(widget)
        layout.addWidget(legendIcon)
        layout.setContentsMargins(4, 0, 4, 0)
        self.setIndexWidget(iconItem.index(), widget)

    def _createRemoveButton(self, row: int, parentItem: qt.QStandardItem | None):
        """Create the remove button for a row in the model"""

        # If the parentItem is None, the remove button is for the X dataset
        if parentItem is None:
            parentItem = self.model().getXParent()
            index = self.model().index(0, 2)
            button = self._getRemoveButton(None, parentItem)
            self.setIndexWidget(index, button)
            return

        # If the parentItem is not None, the remove button is for a Y dataset
        removeItem = parentItem.child(row, 2)
        if removeItem:
            button = self._getRemoveButton(removeItem, parentItem)
            self.setIndexWidget(removeItem.index(), button)

    def _getRemoveButton(
        self, removeItem: qt.QStandardItem | None, parentItem: qt.QStandardItem
    ) -> qt.QToolButton:
        """Return a remove button widget."""
        button = qt.QToolButton(self)
        button.setIcon(icons.getQIcon("remove"))
        button.setStyleSheet("QToolButton { border-radius: 0px; }")
        button.clicked.connect(
            functools.partial(self._removeFile, removeItem, parentItem)
        )
        return button

    def _removeFile(
        self, removeItem: qt.QStandardItem | None, parentItem: qt.QStandardItem
    ):
        """Remove a file from the model and the plot."""

        # If removeItem is None, the file to remove is the X dataset
        if removeItem is None:
            row = 0
            removeItem = self.model().getXParent()

        if removeItem:
            row = removeItem.row()

            # If the parentItem is None, the file to remove is the X dataset
            if parentItem is None:
                parentItem = self.model().getXParent()

            parentItem.removeRow(row)

            if parentItem == self.model().getXParent():
                self.model()._xDataset = None
                self.model()._setXFile()
                index = self.model().index(0, 2)
                self.setIndexWidget(index, None)
                self.model()._plot1D.setXAxisLabel("X")
                self.model()._updateYCurvesWithDefaultX()

    def dragEnterEvent(self, event):
        super().dragEnterEvent(event)
        self.acceptDragEvent(event)

    def dragMoveEvent(self, event):
        dropIndex = self.indexAt(event.pos())
        if (
            dropIndex.isValid()
            and self.model().itemFromIndex(dropIndex).parent() is None
        ):
            self.setDropHighlight("X")
        else:
            self.setDropHighlight("Y")
        event.acceptProposedAction()

    def dragLeaveEvent(self, event):
        super().dragLeaveEvent(event)
        self.setDropHighlight(None)

    def dropEvent(self, event):
        super().dropEvent(event)
        byteString = event.mimeData().data("application/x-silx-uri")
        url = silx.io.url.DataUrl(byteString.data().decode("utf-8"))

        dropIndex = self.indexAt(event.pos())
        if (
            dropIndex.isValid()
            and self.model().itemFromIndex(dropIndex).parent() is None
        ):
            self.setX(url)
        else:
            self.addY(url)

        self.setDropHighlight(None)
        event.acceptProposedAction()

    def acceptDragEvent(self, event):
        """Accept the drag event if the mime data contains a 1D dataset."""

        if event.mimeData().hasFormat("application/x-silx-uri"):
            byteString = event.mimeData().data("application/x-silx-uri")
            url = silx.io.url.DataUrl(byteString.data().decode("utf-8"))
            with silx.io.open(url.file_path()) as file:
                data = file[url.data_path()]
                if silx.io.is_dataset(data) and data.ndim == 1:
                    event.acceptProposedAction()
        else:
            event.ignore()

    def setX(self, url: silx.io.url.DataUrl):
        """Set the X dataset in the model and the plot."""
        targetNode = "X"
        node = self.model().getXParent()
        self.model().addUrl(url, targetNode)
        self._createRemoveButton(node, None)
        self.model()._plot1D.resetZoom()

    def addY(self, url: silx.io.url.DataUrl):
        """Add a Y dataset to the model and the plot."""
        targetNode = "Y"
        node = self.model().getYParent()
        self.model().addUrl(url, targetNode)
        self._createIconWidget(node.rowCount() - 2, node)
        self._createRemoveButton(node.rowCount() - 2, node)
        self.model()._plot1D.resetZoom()

    def clear(self):
        """Clear all datasets from the model and the plot."""
        self.model().clearAll()
        index = self.model().index(0, 2)
        self.setIndexWidget(index, None)

    def setDropHighlight(self, value: str | None):
        """Set the drop highlight for the X and Y datasets."""
        xDropFileItem = self.model().itemFromIndex(
            self.model().sibling(
                0, 1, self.model().indexFromItem(self.model().getXParent())
            )
        )
        xDropFileItem.setData(value == "X", _DROP_HIGHLIGHT_ROLE)
        yDropFileItem = (
            self.model().getYParent().child(self.model().getYParent().rowCount() - 1, 1)
        )
        yDropFileItem.setData(value == "Y", _DROP_HIGHLIGHT_ROLE)


class _DropPlot1D(plot.Plot1D):
    """Plot1D widget for displaying 1D datasets that can accept drops."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAcceptDrops(True)
        self._treeView = None
        self.dropOverlay = DropOverlay(self)

    def setTreeView(self, treeView: qt.QTreeView):
        """Set the TreeView widget for the plot."""
        self._treeView = treeView

    def dragEnterEvent(self, event):
        super().dragEnterEvent(event)
        self._treeView.acceptDragEvent(event)
        if event.isAccepted():
            self._showDropOverlay(event)
            self.dropOverlay.show()

    def dragMoveEvent(self, event):
        super().dragMoveEvent(event)
        self._showDropOverlay(event)

    def dragLeaveEvent(self, event):
        super().dragLeaveEvent(event)
        self.dropOverlay.hide()

    def dropEvent(self, event):
        super().dropEvent(event)
        self.dropOverlay.hide()
        byteString = event.mimeData().data("application/x-silx-uri")
        url = silx.io.url.DataUrl(byteString.data().decode("utf-8"))

        plotArea = self.getWidgetHandle()
        dropPosition = plotArea.mapFrom(self, event.pos())

        plotBounds = self.getPlotBoundsInPixels()
        left, top, width, height = plotBounds
        yAreaTop = top + height

        if dropPosition.y() > yAreaTop:
            self._treeView.setX(url)
        else:
            self._treeView.addY(url)

        self.resetZoom()
        event.acceptProposedAction()

    def setXAxisLabel(self, label: str):
        """Set the label for the X axis."""
        xAxis = self.getXAxis()
        xAxis.setLabel(label)

    def _showDropOverlay(self, event):
        """Show the drop overlay at the drop position."""
        plotArea = self.getWidgetHandle()
        dropPosition = plotArea.mapFrom(self, event.pos())
        offset = plotArea.mapTo(self, qt.QPoint(0, 0))

        plotBounds = self.getPlotBoundsInPixels()
        left, top, width, height = plotBounds
        yAreaTop = top + height

        if dropPosition.y() > yAreaTop:
            rect = qt.QRect(left + offset.x(), yAreaTop + offset.y(), width, 50)
        else:
            rect = qt.QRect(left + offset.x(), top + offset.y(), width, height)

        self.dropOverlay.setGeometry(rect)


class _PlotToolBar(qt.QToolBar):
    """Toolbar widget for the plot."""

    def __init__(self, parent=None):
        super().__init__(parent)

    def addClearAction(self, treeView: qt.QTreeView):
        """Add the clear action to the toolbar."""
        icon = self.style().standardIcon(qt.QStyle.SP_TrashIcon)
        clearAction = qt.QAction(icon, "Clear All", self)
        clearAction.triggered.connect(treeView.clear)
        self.addAction(clearAction)


class DropOverlay(qt.QWidget):
    """Overlay widget for displaying drop zones on the plot."""

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setAttribute(qt.Qt.WA_TransparentForMouseEvents)
        self.setAttribute(qt.Qt.WA_NoSystemBackground)
        self.hide()

    def paintEvent(self, event):
        """Paint the overlay."""
        painter = qt.QPainter(self)
        painter.setRenderHint(qt.QPainter.Antialiasing)
        brush_color = qt.QColor(0, 0, 0, 50)
        painter.setBrush(qt.QBrush(brush_color))
        painter.drawRect(self.rect())


class CustomPlotSelectionWindow(qt.QMainWindow):
    """A customized plot selection window allowing the user to select and display 1D data sets."""

    sigVisibilityChanged = qt.Signal(bool)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setWindowTitle("Plot selection")

        self._plot1D = _DropPlot1D()
        model = _FileListModel(self._plot1D)

        self._treeView = _DropTreeView(model, self)
        self._plot1D.setTreeView(self._treeView)

        centralWidget = qt.QSplitter()

        centralWidget.addWidget(self._treeView)
        centralWidget.addWidget(self._plot1D)

        centralWidget.setCollapsible(0, False)
        centralWidget.setCollapsible(1, False)
        centralWidget.setStretchFactor(1, 1)

        self.setCentralWidget(centralWidget)

        toolbar = _PlotToolBar(self)
        toolbar.addClearAction(self._treeView)
        self.addToolBar(qt.Qt.TopToolBarArea, toolbar)

    def getPlot1D(self) -> _DropPlot1D:
        """Return the plot widget."""
        return self._plot1D

    def getTreeView(self) -> _DropTreeView:
        """Return the tree view widget."""
        return self._treeView

    def showEvent(self, event):
        super().showEvent(event)
        self.sigVisibilityChanged.emit(True)

    def hideEvent(self, event):
        super().hideEvent(event)
        self.sigVisibilityChanged.emit(False)

    def setX(self, dataUrl: silx.io.url.DataUrl):
        """Set the X dataset in the model and the plot."""
        self._treeView.setX(dataUrl)

    def addY(self, dataUrl: silx.io.url.DataUrl):
        """Add a Y dataset to the model and the plot."""
        self._treeView.addY(dataUrl)



# /*##########################################################################
#
# Copyright (c) 20023 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""
PlotToolButton to measure a distance in a plot
"""

__authors__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "30/10/2023"


import logging
import numpy
import weakref
import typing

from silx.gui import icons

from .PlotToolButton import PlotToolButton

from silx.gui.plot.tools.roi import RegionOfInterestManager
from silx.gui.plot.items.roi import LineROI
from silx.gui.plot import items


_logger = logging.getLogger(__name__)


class _RulerROI(LineROI):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._formatFunction: typing.Optional[
            typing.Callable[
                [numpy.ndarray, numpy.ndarray], str
            ]
        ] = None
        self.setColor("#001122")  # Only there to trig updateStyle

    def registerFormatFunction(
        self,
        fct: typing.Callable[
            [numpy.ndarray, numpy.ndarray], str
        ],
    ):
        """Register a function for the formatting of the label"""
        self._formatFunction = fct

    def _updatedStyle(self, event, style: items.CurveStyle):
        style = items.CurveStyle(
            color="red",
            gapcolor="white",
            linestyle=(0, (5, 5)),
            linewidth=style.getLineWidth())
        LineROI._updatedStyle(self, event, style)
        self._handleLabel.setColor("black")
        self._handleLabel.setBackgroundColor("#FFFFFF60")
        self._handleLabel.setZValue(1000)

    def setEndPoints(self, startPoint: numpy.ndarray, endPoint: numpy.ndarray):
        super().setEndPoints(startPoint=startPoint, endPoint=endPoint)
        if self._formatFunction is not None:
            ruler_text = self._formatFunction(
                startPoint=startPoint, endPoint=endPoint
            )
            self._updateText(ruler_text)


class RulerToolButton(PlotToolButton):
    """
    Button to active measurement between two point of the plot

    An instance of `RulerToolButton` can be added to a plot toolbar like:
    .. code-block:: python

        plot = Plot2D()

        rulerButton = RulerToolButton(parent=plot, plot=plot)
        plot.toolBar().addWidget(rulerButton)
    """

    def __init__(
        self,
        parent=None,
        plot=None,
    ):
        super().__init__(parent=parent, plot=plot)
        self.setCheckable(True)
        self._roiManager = None
        self.__lastRoiCreated = None
        self.setIcon(icons.getQIcon("ruler"))
        self.toggled.connect(self._callback)
        self._connectPlot(plot)

    def setPlot(self, plot):
        return super().setPlot(plot)

    @property
    def _lastRoiCreated(self):
        if self.__lastRoiCreated is None:
            return None
        return self.__lastRoiCreated()

    def _callback(self, *args, **kwargs):
        if not self._roiManager:
            return
        if self._lastRoiCreated is not None:
            self._lastRoiCreated.setVisible(self.isChecked())
        if self.isChecked():
            self._roiManager.start(_RulerROI, self)
            self.__interactiveModeStarted(self._roiManager)
        else:
            source = self._roiManager.getInteractionSource()
            if source is self:
                self._roiManager.stop()

    def __interactiveModeStarted(self, roiManager):
        roiManager.sigInteractiveModeFinished.connect(self.__interactiveModeFinished)

    def __interactiveModeFinished(self):
        roiManager = self._roiManager
        if roiManager is not None:
            roiManager.sigInteractiveModeFinished.disconnect(
                self.__interactiveModeFinished
            )
        self.setChecked(False)

    def _connectPlot(self, plot):
        """
        Called when the plot is connected to the widget

        :param plot: :class:`.PlotWidget` instance
        """
        if plot is None:
            return
        self._roiManager = RegionOfInterestManager(plot)
        self._roiManager.sigRoiAdded.connect(self._registerCurrentROI)

    def _disconnectPlot(self, plot):
        if plot and self._lastRoiCreated is not None:
            self._roiManager.removeRoi(self._lastRoiCreated)
            self.__lastRoiCreated = None
        return super()._disconnectPlot(plot)

    def _registerCurrentROI(self, currentRoi):
        if self._lastRoiCreated is None:
            self.__lastRoiCreated = weakref.ref(currentRoi)
            self._lastRoiCreated.registerFormatFunction(self.buildDistanceText)
        elif currentRoi is not self._lastRoiCreated and self._roiManager is not None:
            self._roiManager.removeRoi(self._lastRoiCreated)
            currentRoi.registerFormatFunction(self.buildDistanceText)
            self.__lastRoiCreated = weakref.ref(currentRoi)

    def buildDistanceText(self, startPoint: numpy.ndarray, endPoint: numpy.ndarray) -> str:
        """
        Define the text to be displayed by the ruler.

        It can be redefine to modify precision or handle other parameters
        (handling pixel size to display metric distance, display distance
        on each distance - for non-square pixels...)
        """
        distance = numpy.linalg.norm(endPoint - startPoint)
        return f"{distance: .1f}px"



# /*##########################################################################
#
# Copyright (c) 2016-2023 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Basic tests for PlotWidget with 'none' backend"""

__authors__ = ["T. Vincent"]
__license__ = "MIT"
__date__ = "17/01/2018"


import unittest
from functools import reduce
from silx.utils.testutils import ParametricTestCase

import numpy

import silx
from silx.gui.colors import rgba
from silx.gui.plot.PlotWidget import PlotWidget
from silx.gui.plot.items.histogram import _getHistogramCurve, _computeEdges


class TestPlot(unittest.TestCase):
    """Basic tests of Plot without backend"""

    def testPlotTitleLabels(self):
        """Create a Plot and set the labels"""

        plot = PlotWidget(backend="none")

        title, xlabel, ylabel = "the title", "x label", "y label"
        plot.setGraphTitle(title)
        plot.getXAxis().setLabel(xlabel)
        plot.getYAxis().setLabel(ylabel)

        self.assertEqual(plot.getGraphTitle(), title)
        self.assertEqual(plot.getXAxis().getLabel(), xlabel)
        self.assertEqual(plot.getYAxis().getLabel(), ylabel)

    def testAddNoRemove(self):
        """add objects to the Plot"""

        plot = PlotWidget(backend="none")
        plot.addCurve(x=(1, 2, 3), y=(3, 2, 1))
        plot.addImage(numpy.arange(100.0).reshape(10, -1))
        plot.addShape(
            numpy.array((1.0, 10.0)), numpy.array((10.0, 10.0)), shape="rectangle"
        )
        plot.addXMarker(10.0)


class TestPlotRanges(ParametricTestCase):
    """Basic tests of Plot data ranges without backend"""

    _getValidValues = {
        True: lambda ar: ar > 0,
        False: lambda ar: numpy.ones(shape=ar.shape, dtype=bool),
    }

    @staticmethod
    def _getRanges(arrays, are_logs):
        gen = (
            TestPlotRanges._getValidValues[is_log](ar)
            for (ar, is_log) in zip(arrays, are_logs)
        )
        indices = numpy.where(reduce(numpy.logical_and, gen))[0]
        if len(indices) > 0:
            ranges = [(ar[indices[0]], ar[indices[-1]]) for ar in arrays]
        else:
            ranges = [None] * len(arrays)

        return ranges

    @staticmethod
    def _getRangesMinmax(ranges):
        # TODO : error if None in ranges.
        rangeMin = numpy.min([rng[0] for rng in ranges])
        rangeMax = numpy.max([rng[1] for rng in ranges])
        return rangeMin, rangeMax

    def testDataRangeNoPlot(self):
        """empty plot data range"""

        plot = PlotWidget(backend="none")

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                self.assertIsNone(dataRange.x)
                self.assertIsNone(dataRange.y)
                self.assertIsNone(dataRange.yright)

    def testDataRangeLeft(self):
        """left axis range"""

        plot = PlotWidget(backend="none")

        xData = numpy.arange(10) - 4.9  # range : -4.9 , 4.1
        yData = numpy.arange(10) - 6.9  # range : -6.9 , 2.1

        plot.addCurve(x=xData, y=yData, legend="plot_0", yaxis="left")

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRange, yRange = self._getRanges([xData, yData], [logX, logY])
                self.assertSequenceEqual(dataRange.x, xRange)
                self.assertSequenceEqual(dataRange.y, yRange)
                self.assertIsNone(dataRange.yright)

    def testDataRangeRight(self):
        """right axis range"""

        plot = PlotWidget(backend="none")
        xData = numpy.arange(10) - 4.9  # range : -4.9 , 4.1
        yData = numpy.arange(10) - 6.9  # range : -6.9 , 2.1
        plot.addCurve(x=xData, y=yData, legend="plot_0", yaxis="right")

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRange, yRange = self._getRanges([xData, yData], [logX, logY])
                self.assertSequenceEqual(dataRange.x, xRange)
                self.assertIsNone(dataRange.y)
                self.assertSequenceEqual(dataRange.yright, yRange)

    def testDataRangeImage(self):
        """image data range"""

        origin = (-10, 25)
        scale = (3.0, 8.0)
        image = numpy.arange(100.0).reshape(20, 5)

        plot = PlotWidget(backend="none")
        plot.addImage(image, origin=origin, scale=scale)

        xRange = numpy.array([0.0, image.shape[1] * scale[0]]) + origin[0]
        yRange = numpy.array([0.0, image.shape[0] * scale[1]]) + origin[1]

        ranges = {
            (False, False): (xRange, yRange),
            (True, False): (None, None),
            (True, True): (None, None),
            (False, True): (None, None),
        }

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRange, yRange = ranges[logX, logY]
                self.assertTrue(
                    numpy.array_equal(dataRange.x, xRange),
                    msg="{0} != {1}".format(dataRange.x, xRange),
                )
                self.assertTrue(
                    numpy.array_equal(dataRange.y, yRange),
                    msg="{0} != {1}".format(dataRange.y, yRange),
                )
                self.assertIsNone(dataRange.yright)

    def testDataRangeLeftRight(self):
        """right+left axis range"""

        plot = PlotWidget(backend="none")

        xData_l = numpy.arange(10) - 0.9  # range : -0.9 , 8.1
        yData_l = numpy.arange(10) - 1.9  # range : -1.9 , 7.1
        plot.addCurve(x=xData_l, y=yData_l, legend="plot_l", yaxis="left")

        xData_r = numpy.arange(10) - 4.9  # range : -4.9 , 4.1
        yData_r = numpy.arange(10) - 6.9  # range : -6.9 , 2.1
        plot.addCurve(x=xData_r, y=yData_r, legend="plot_r", yaxis="right")

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRangeL, yRangeL = self._getRanges([xData_l, yData_l], [logX, logY])
                xRangeR, yRangeR = self._getRanges([xData_r, yData_r], [logX, logY])
                xRangeLR = self._getRangesMinmax([xRangeL, xRangeR])
                self.assertSequenceEqual(dataRange.x, xRangeLR)
                self.assertSequenceEqual(dataRange.y, yRangeL)
                self.assertSequenceEqual(dataRange.yright, yRangeR)

    def testDataRangeCurveImage(self):
        """right+left+image axis range"""

        # overlapping ranges :
        # image sets x min and y max
        # plot_left sets y min
        # plot_right sets x max (and yright)
        plot = PlotWidget(backend="none")

        origin = (-10, 5)
        scale = (3.0, 8.0)
        image = numpy.arange(100.0).reshape(20, 5)

        plot.addImage(image, origin=origin, scale=scale, legend="image")

        xData_l = numpy.arange(10) - 0.9  # range : -0.9 , 8.1
        yData_l = numpy.arange(10) - 1.9  # range : -1.9 , 7.1
        plot.addCurve(x=xData_l, y=yData_l, legend="plot_l", yaxis="left")

        xData_r = numpy.arange(10) + 4.1  # range : 4.1 , 13.1
        yData_r = numpy.arange(10) - 0.9  # range : -0.9 , 8.1
        plot.addCurve(x=xData_r, y=yData_r, legend="plot_r", yaxis="right")

        imgXRange = numpy.array([0.0, image.shape[1] * scale[0]]) + origin[0]
        imgYRange = numpy.array([0.0, image.shape[0] * scale[1]]) + origin[1]

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRangeL, yRangeL = self._getRanges([xData_l, yData_l], [logX, logY])
                xRangeR, yRangeR = self._getRanges([xData_r, yData_r], [logX, logY])
                if logX or logY:
                    xRangeLR = self._getRangesMinmax([xRangeL, xRangeR])
                else:
                    xRangeLR = self._getRangesMinmax([xRangeL, xRangeR, imgXRange])
                    yRangeL = self._getRangesMinmax([yRangeL, imgYRange])
                self.assertSequenceEqual(dataRange.x, xRangeLR)
                self.assertSequenceEqual(dataRange.y, yRangeL)
                self.assertSequenceEqual(dataRange.yright, yRangeR)

    def testDataRangeImageNegativeScaleX(self):
        """image data range, negative scale"""

        origin = (-10, 25)
        scale = (-3.0, 8.0)
        image = numpy.arange(100.0).reshape(20, 5)

        plot = PlotWidget(backend="none")
        plot.addImage(image, origin=origin, scale=scale)

        xRange = numpy.array([0.0, image.shape[1] * scale[0]]) + origin[0]
        xRange.sort()  # negative scale!
        yRange = numpy.array([0.0, image.shape[0] * scale[1]]) + origin[1]

        ranges = {
            (False, False): (xRange, yRange),
            (True, False): (None, None),
            (True, True): (None, None),
            (False, True): (None, None),
        }

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRange, yRange = ranges[logX, logY]
                self.assertTrue(
                    numpy.array_equal(dataRange.x, xRange),
                    msg="{0} != {1}".format(dataRange.x, xRange),
                )
                self.assertTrue(
                    numpy.array_equal(dataRange.y, yRange),
                    msg="{0} != {1}".format(dataRange.y, yRange),
                )
                self.assertIsNone(dataRange.yright)

    def testDataRangeImageNegativeScaleY(self):
        """image data range, negative scale"""

        origin = (-10, 25)
        scale = (3.0, -8.0)
        image = numpy.arange(100.0).reshape(20, 5)

        plot = PlotWidget(backend="none")
        plot.addImage(image, origin=origin, scale=scale)

        xRange = numpy.array([0.0, image.shape[1] * scale[0]]) + origin[0]
        yRange = numpy.array([0.0, image.shape[0] * scale[1]]) + origin[1]
        yRange.sort()  # negative scale!

        ranges = {
            (False, False): (xRange, yRange),
            (True, False): (None, None),
            (True, True): (None, None),
            (False, True): (None, None),
        }

        for logX, logY in (
            (False, False),
            (True, False),
            (True, True),
            (False, True),
            (False, False),
        ):
            with self.subTest(logX=logX, logY=logY):
                plot.getXAxis()._setLogarithmic(logX)
                plot.getYAxis()._setLogarithmic(logY)
                dataRange = plot.getDataRange()
                xRange, yRange = ranges[logX, logY]
                self.assertTrue(
                    numpy.array_equal(dataRange.x, xRange),
                    msg="{0} != {1}".format(dataRange.x, xRange),
                )
                self.assertTrue(
                    numpy.array_equal(dataRange.y, yRange),
                    msg="{0} != {1}".format(dataRange.y, yRange),
                )
                self.assertIsNone(dataRange.yright)

    def testDataRangeHiddenCurve(self):
        """curves with a hidden curve"""
        plot = PlotWidget(backend="none")
        plot.addCurve((0, 1), (0, 1), legend="shown")
        plot.addCurve((0, 1, 2), (5, 5, 5), legend="hidden")
        range1 = plot.getDataRange()
        self.assertEqual(range1.x, (0, 2))
        self.assertEqual(range1.y, (0, 5))
        plot.hideCurve("hidden")
        range2 = plot.getDataRange()
        self.assertEqual(range2.x, (0, 1))
        self.assertEqual(range2.y, (0, 1))


class TestPlotGetCurveImage(unittest.TestCase):
    """Test of plot getCurve and getImage methods"""

    def testGetCurve(self):
        """PlotWidget.getCurve and Plot.getActiveCurve tests"""

        plot = PlotWidget(backend="none")

        # No curve
        curve = plot.getCurve()
        self.assertIsNone(curve)  # No curve

        plot.setActiveCurveHandling(True)
        plot.addCurve(x=(0, 1), y=(0, 1), legend="curve 0")
        plot.addCurve(x=(0, 1), y=(0, 1), legend="curve 1")
        plot.addCurve(x=(0, 1), y=(0, 1), legend="curve 2")
        plot.setActiveCurve("curve 0")

        # Active curve
        active = plot.getActiveCurve()
        self.assertEqual(active.getName(), "curve 0")
        curve = plot.getCurve()
        self.assertEqual(curve.getName(), "curve 0")

        # No active curve and curves
        plot.setActiveCurveHandling(False)
        active = plot.getActiveCurve()
        self.assertIsNone(active)  # No active curve
        curve = plot.getCurve()
        self.assertEqual(curve.getName(), "curve 2")  # Last added curve

        # Last curve hidden
        plot.hideCurve("curve 2", True)
        curve = plot.getCurve()
        self.assertEqual(curve.getName(), "curve 1")  # Last added curve

        # All curves hidden
        plot.hideCurve("curve 1", True)
        plot.hideCurve("curve 0", True)
        curve = plot.getCurve()
        self.assertIsNone(curve)

    def testGetCurveOldApi(self):
        """old API PlotWidget.getCurve and Plot.getActiveCurve tests"""

        plot = PlotWidget(backend="none")

        # No curve
        curve = plot.getCurve()
        self.assertIsNone(curve)  # No curve

        plot.setActiveCurveHandling(True)
        x = numpy.arange(10.0).astype(numpy.float32)
        y = x * x
        plot.addCurve(x=x, y=y, legend="curve 0", info=["whatever"])
        plot.addCurve(x=x, y=2 * x, legend="curve 1", info="anything")
        plot.setActiveCurve("curve 0")

        # Active curve (4 elements)
        xOut, yOut, legend, info = plot.getActiveCurve()[:4]
        self.assertEqual(legend, "curve 0")
        self.assertTrue(numpy.allclose(xOut, x), "curve 0 wrong x data")
        self.assertTrue(numpy.allclose(yOut, y), "curve 0 wrong y data")

        # Active curve (5 elements)
        xOut, yOut, legend, info, params = plot.getCurve("curve 1")
        self.assertEqual(legend, "curve 1")
        self.assertEqual(info, "anything")
        self.assertTrue(numpy.allclose(xOut, x), "curve 1 wrong x data")
        self.assertTrue(numpy.allclose(yOut, 2 * x), "curve 1 wrong y data")

    def testGetImage(self):
        """PlotWidget.getImage and PlotWidget.getActiveImage tests"""

        plot = PlotWidget(backend="none")

        # No image
        image = plot.getImage()
        self.assertIsNone(image)

        plot.addImage(((0, 1), (2, 3)), legend="image 0")
        plot.addImage(((0, 1), (2, 3)), legend="image 1")

        # Active image
        active = plot.getActiveImage()
        self.assertEqual(active.getName(), "image 0")
        image = plot.getImage()
        self.assertEqual(image.getName(), "image 0")

        # No active image
        plot.addImage(((0, 1), (2, 3)), legend="image 2")
        plot.setActiveImage(None)
        active = plot.getActiveImage()
        self.assertIsNone(active)
        image = plot.getImage()
        self.assertEqual(image.getName(), "image 2")

        # Active image
        plot.setActiveImage("image 1")
        active = plot.getActiveImage()
        self.assertEqual(active.getName(), "image 1")
        image = plot.getImage()
        self.assertEqual(image.getName(), "image 1")

    def testGetImageOldApi(self):
        """PlotWidget.getImage and PlotWidget.getActiveImage old API tests"""

        plot = PlotWidget(backend="none")

        # No image
        image = plot.getImage()
        self.assertIsNone(image)

        image = numpy.arange(10).astype(numpy.float32)
        image.shape = 5, 2

        plot.addImage(image, legend="image 0", info=["Hi!"])

        # Active image
        data, legend, info, something, params = plot.getActiveImage()
        self.assertEqual(legend, "image 0")
        self.assertEqual(info, ["Hi!"])
        self.assertTrue(numpy.allclose(data, image), "image 0 data not correct")

    def testGetAllImages(self):
        """PlotWidget.getAllImages test"""

        plot = PlotWidget(backend="none")

        # No image
        images = plot.getAllImages()
        self.assertEqual(len(images), 0)

        # 2 images
        data = numpy.arange(100).reshape(10, 10)
        plot.addImage(data, legend="1")
        plot.addImage(data, origin=(10, 10), legend="2")
        images = plot.getAllImages(just_legend=True)
        self.assertEqual(list(images), ["1", "2"])
        images = plot.getAllImages(just_legend=False)
        self.assertEqual(len(images), 2)
        self.assertEqual(images[0].getName(), "1")
        self.assertEqual(images[1].getName(), "2")


class TestPlotAddScatter(unittest.TestCase):
    """Test of plot addScatter"""

    def testAddGetScatter(self):
        plot = PlotWidget(backend="none")

        # No curve
        scatter = plot._getItem(kind="scatter")
        self.assertIsNone(scatter)  # No curve

        plot.addScatter(x=(0, 1), y=(0, 1), value=(0, 1), legend="scatter 0")
        plot.addScatter(x=(0, 1), y=(0, 1), value=(0, 1), legend="scatter 1")
        plot.addScatter(x=(0, 1), y=(0, 1), value=(0, 1), legend="scatter 2")
        plot.setActiveScatter("scatter 0")

        # Active scatter
        active = plot.getActiveScatter()
        self.assertEqual(active.getName(), "scatter 0")

        # check default values
        self.assertAlmostEqual(active.getSymbolSize(), active._DEFAULT_SYMBOL_SIZE)
        self.assertEqual(active.getSymbol(), "o")
        self.assertAlmostEqual(active.getAlpha(), 1.0)

        # modify parameters
        active.setSymbolSize(20.5)
        active.setSymbol("d")
        active.setAlpha(0.777)

        s0 = plot.getScatter("scatter 0")

        self.assertAlmostEqual(s0.getSymbolSize(), 20.5)
        self.assertEqual(s0.getSymbol(), "d")
        self.assertAlmostEqual(s0.getAlpha(), 0.777)

        scatter1 = plot._getItem(kind="scatter", legend="scatter 1")
        self.assertEqual(scatter1.getName(), "scatter 1")

    def testGetAllScatters(self):
        """PlotWidget.getAllImages test"""

        plot = PlotWidget(backend="none")

        items = plot.getItems()
        self.assertEqual(len(items), 0)

        plot.addScatter(x=(0, 1), y=(0, 1), value=(0, 1), legend="scatter 0")
        plot.addScatter(x=(0, 1), y=(0, 1), value=(0, 1), legend="scatter 1")
        plot.addScatter(x=(0, 1), y=(0, 1), value=(0, 1), legend="scatter 2")

        items = plot.getItems()
        self.assertEqual(len(items), 3)
        self.assertEqual(items[0].getName(), "scatter 0")
        self.assertEqual(items[1].getName(), "scatter 1")
        self.assertEqual(items[2].getName(), "scatter 2")


class TestPlotHistogram(unittest.TestCase):
    """Basic tests for histogram."""

    def testEdges(self):
        x = numpy.array([0, 1, 2])
        edgesRight = numpy.array([0, 1, 2, 3])
        edgesLeft = numpy.array([-1, 0, 1, 2])
        edgesCenter = numpy.array([-0.5, 0.5, 1.5, 2.5])

        # testing x values for right
        edges = _computeEdges(x, "right")
        numpy.testing.assert_array_equal(edges, edgesRight)

        edges = _computeEdges(x, "center")
        numpy.testing.assert_array_equal(edges, edgesCenter)

        edges = _computeEdges(x, "left")
        numpy.testing.assert_array_equal(edges, edgesLeft)

    def testHistogramCurve(self):
        y = numpy.array([3, 2, 5])
        edges = numpy.array([0, 1, 2, 3])

        xHisto, yHisto = _getHistogramCurve(y, edges)
        numpy.testing.assert_array_equal(yHisto, numpy.array([3, 3, 2, 2, 5, 5]))

        y = numpy.array([-3, 2, 5, 0])
        edges = numpy.array([-2, -1, 0, 1, 2])
        xHisto, yHisto = _getHistogramCurve(y, edges)
        numpy.testing.assert_array_equal(
            yHisto, numpy.array([-3, -3, 2, 2, 5, 5, 0, 0])
        )


def testSetDefaultColors(qWidgetFactory):
    """Basic test of PlotWidget.get|setDefaultColors"""
    plot = qWidgetFactory(PlotWidget)

    # By default using config
    assert numpy.array_equal(
        plot.getDefaultColors(), silx.config.DEFAULT_PLOT_CURVE_COLORS
    )

    # Use own colors
    colors = "red", "green", "blue"
    plot.setDefaultColors(colors)
    assert plot.getDefaultColors() == colors

    # Reset to default
    plot.setDefaultColors(None)
    assert numpy.array_equal(
        plot.getDefaultColors(), silx.config.DEFAULT_PLOT_CURVE_COLORS
    )


def testSetDefaultColorsAddCurve(qWidgetFactory):
    """Test that PlotWidget.setDefaultColors reset color index"""
    plot = qWidgetFactory(PlotWidget)

    plot.addCurve((0, 1), (0, 0), legend="curve0")
    plot.addCurve((0, 1), (1, 1), legend="curve1")
    plot.addCurve((0, 1), (2, 2), legend="curve2")

    colors = "#123456", "#abcdef"
    plot.setDefaultColors(colors)
    assert plot.getDefaultColors() == colors

    # Check that the color index is reset
    curve = plot.addCurve((1, 2), (0, 1), legend="newcurve")
    assert curve.getColor() == rgba(colors[0])


def testDefaultColorsUpdateConfig(qWidgetFactory):
    """Test that color index is reset if needed when default colors config is updated"""
    plot = qWidgetFactory(PlotWidget)

    plot.addCurve((0, 1), (0, 0), legend="curve0")
    plot.addCurve((0, 1), (1, 1), legend="curve1")
    plot.addCurve((0, 1), (2, 2), legend="curve2")

    previous_colors = silx.config.DEFAULT_PLOT_CURVE_COLORS
    try:
        colors = "#123456", "#abcdef"
        silx.config.DEFAULT_PLOT_CURVE_COLORS = colors
        assert plot.getDefaultColors() == colors

        # Check that the color index is reset
        curve = plot.addCurve((1, 2), (0, 1), legend="newcurve")
        assert curve.getColor() == rgba(colors[0])

    finally:
        silx.config.DEFAULT_PLOT_CURVE_COLORS = previous_colors



# /*##########################################################################
#
# Copyright (c) 2016-2024 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Basic tests for ColormapDialog"""

__authors__ = ["T. Vincent"]
__license__ = "MIT"
__date__ = "09/11/2018"


import pytest

from silx.gui import qt
from silx.gui.dialog import ColormapDialog
from silx.gui.colors import Colormap, preferredColormaps
from silx.gui.plot.items.image import ImageData

import numpy


def testGUIEdition(qWidgetFactory):
    """Make sure the colormap is correctly edited and also that the
    modification are correctly updated if an other colormapdialog is
    editing the same colormap"""
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog.setColormap(colormap)
    dialog2 = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog2.setColormap(colormap)

    dialog._comboBoxColormap._setCurrentName("red")
    dialog._comboBoxNormalization.setCurrentIndex(
        dialog._comboBoxNormalization.findData(Colormap.LOGARITHM)
    )
    assert colormap.getName() == "red"
    assert dialog.getColormap().getName() == "red"
    assert colormap.getNormalization() == "log"
    assert colormap.getVMin() == 10
    assert colormap.getVMax() == 20
    # checked second colormap dialog
    assert dialog2._comboBoxColormap.getCurrentName() == "red"
    assert dialog2._comboBoxNormalization.currentData() == Colormap.LOGARITHM
    assert int(dialog2._minValue.getValue()) == 10
    assert int(dialog2._maxValue.getValue()) == 20


def testGUIModalOk(qapp, qapp_utils, qWidgetFactory):
    """Make sure the colormap is modified if gone through accept"""
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    assert colormap.isAutoscale() is False
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog.setModal(True)
    qapp.processEvents()

    dialog.setColormap(colormap)
    assert colormap.getVMin() is not None
    dialog._minValue.sigAutoScaleChanged.emit(True)
    assert colormap.getVMin() is None
    dialog._maxValue.sigAutoScaleChanged.emit(True)
    qapp_utils.mouseClick(
        widget=dialog._buttonsModal.button(qt.QDialogButtonBox.Ok),
        button=qt.Qt.LeftButton,
    )
    assert colormap.getVMin() is None
    assert colormap.getVMax() is None
    assert colormap.isAutoscale() is True


def testGUIModalCancel(qapp, qapp_utils, qWidgetFactory):
    """Make sure the colormap is not modified if gone through reject"""
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    assert colormap.isAutoscale() is False
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog.setModal(True)
    qapp.processEvents()

    dialog.setColormap(colormap)
    assert colormap.getVMin() is not None
    dialog._minValue.sigAutoScaleChanged.emit(True)
    assert colormap.getVMin() is None
    qapp_utils.mouseClick(
        widget=dialog._buttonsModal.button(qt.QDialogButtonBox.Cancel),
        button=qt.Qt.LeftButton,
    )
    assert colormap.getVMin() is not None


def testGUIModalClose(qapp, qapp_utils, qWidgetFactory):
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    assert colormap.isAutoscale() is False
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog.setModal(False)
    qapp.processEvents()

    dialog.setColormap(colormap)
    assert colormap.getVMin() is not None
    dialog._minValue.sigAutoScaleChanged.emit(True)
    assert colormap.getVMin() is None
    qapp_utils.mouseClick(
        widget=dialog._buttonsNonModal.button(qt.QDialogButtonBox.Close),
        button=qt.Qt.LeftButton,
    )
    assert colormap.getVMin() is None


def testGUIModalReset(qapp, qapp_utils, qWidgetFactory):
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    assert colormap.isAutoscale() is False
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog.setModal(False)
    dialog.show()
    qapp.processEvents()
    dialog.setColormap(colormap)
    assert colormap.getVMin() is not None
    dialog._minValue.sigAutoScaleChanged.emit(True)
    assert colormap.getVMin() is None
    qapp_utils.mouseClick(
        widget=dialog._buttonsNonModal.button(qt.QDialogButtonBox.Reset),
        button=qt.Qt.LeftButton,
    )
    assert colormap.getVMin() is not None
    dialog.close()


def testGUIClose(qapp, qWidgetFactory):
    """Make sure the colormap is modify if go through reject"""
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    assert colormap.isAutoscale() is False
    qapp.processEvents()

    dialog.setColormap(colormap)
    assert colormap.getVMin() is not None
    dialog._minValue.sigAutoScaleChanged.emit(True)
    assert colormap.getVMin() is None
    dialog.close()
    qapp.processEvents()
    assert colormap.getVMin() is None


@pytest.mark.parametrize("norm", Colormap.NORMALIZATIONS)
@pytest.mark.parametrize("autoscale", (True, False))
def testSetColormapIsCorrect(norm, autoscale, qapp, qWidgetFactory):
    """Make sure the interface fir the colormap when set a new colormap"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)

    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    colormap.setName("red")
    if autoscale is True:
        colormap.setVRange(None, None)
    else:
        colormap.setVRange(11, 101)
    colormap.setNormalization(norm)
    dialog.setColormap(colormap)
    qapp.processEvents()

    assert dialog._comboBoxNormalization.currentData() == norm
    assert dialog._comboBoxColormap.getCurrentName() == "red"
    assert dialog._minValue.isAutoChecked() == autoscale
    assert dialog._maxValue.isAutoChecked() == autoscale
    if autoscale is False:
        assert dialog._minValue.getValue() == 11
        assert dialog._maxValue.getValue() == 101
        assert dialog._minValue.isEnabled()
        assert dialog._maxValue.isEnabled()
    else:
        assert dialog._minValue._numVal.isReadOnly()
        assert dialog._maxValue._numVal.isReadOnly()


def testColormapDel(qapp, qWidgetFactory):
    """Check behavior if the colormap has been deleted outside. For now
    we make sure the colormap is still running and nothing more"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormap = Colormap(name="gray")
    dialog.setColormap(colormap)
    qapp.processEvents()

    colormap = None
    assert dialog.getColormap() is None
    dialog._comboBoxColormap._setCurrentName("blue")


def testColormapEditedOutside(qapp, qWidgetFactory):
    """Make sure the GUI is still up to date if the colormap is modified
    outside"""
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    dialog.setColormap(colormap)
    qapp.processEvents()

    colormap.setName("red")
    assert dialog._comboBoxColormap.getCurrentName() == "red"
    colormap.setNormalization(Colormap.LOGARITHM)
    assert dialog._comboBoxNormalization.currentData() == Colormap.LOGARITHM
    colormap.setVRange(11, 201)
    assert dialog._minValue.getValue() == 11
    assert dialog._maxValue.getValue() == 201
    assert not (dialog._minValue._numVal.isReadOnly())
    assert not (dialog._maxValue._numVal.isReadOnly())
    assert not (dialog._minValue.isAutoChecked())
    assert not (dialog._maxValue.isAutoChecked())
    colormap.setVRange(None, None)
    qapp.processEvents()

    assert dialog._minValue._numVal.isReadOnly()
    assert dialog._maxValue._numVal.isReadOnly()
    assert dialog._minValue.isAutoChecked()
    assert dialog._maxValue.isAutoChecked()


def testSetColormapScenario(qWidgetFactory):
    """Test of a simple scenario of a colormap dialog editing several
    colormap"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormap = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    colormap1 = Colormap(name="gray", vmin=10.0, vmax=20.0, normalization="linear")
    colormap2 = Colormap(name="red", vmin=10.0, vmax=20.0, normalization="log")
    colormap3 = Colormap(name="blue", vmin=None, vmax=None, normalization="linear")

    dialog.setColormap(colormap)
    dialog.setColormap(colormap1)
    del colormap1
    dialog.setColormap(colormap2)
    del colormap2
    dialog.setColormap(colormap3)
    del colormap3


def testNotPreferredColormap(qapp, qWidgetFactory):
    """Test that the colormapEditor is able to edit a colormap which is not
    part of the 'prefered colormap'
    """

    def getFirstNotPreferredColormap():
        cms = Colormap.getSupportedColormaps()
        preferred = preferredColormaps()
        for cm in cms:
            if cm not in preferred:
                return cm
        return None

    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormapName = getFirstNotPreferredColormap()
    assert colormapName is not None
    colormap = Colormap(name=colormapName)
    dialog.setColormap(colormap)
    qapp.processEvents()

    cb = dialog._comboBoxColormap
    assert cb.getCurrentName() == colormapName
    cb.setCurrentIndex(0)
    index = cb.findLutName(colormapName)
    assert index != 0  # if 0 then the rest of the test has no sense
    cb.setCurrentIndex(index)
    assert cb.getCurrentName() == colormapName


def testColormapEditableMode(qWidgetFactory):
    """Test that the colormapDialog is correctly updated when changing the
    colormap editable status"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormap = Colormap(normalization="linear", vmin=1.0, vmax=10.0)

    dialog.setColormap(colormap)

    for editable in (True, False):
        colormap.setEditable(editable)
        assert dialog._comboBoxColormap.isEnabled() is editable
        assert dialog._minValue.isEnabled() is editable
        assert dialog._maxValue.isEnabled() is editable
        assert dialog._comboBoxNormalization.isEnabled() is editable

    # Make sure the reset button is also set to enable when edition mode is
    # False
    dialog.setModal(False)
    colormap.setEditable(True)
    dialog._comboBoxNormalization.setCurrentIndex(
        dialog._comboBoxNormalization.findData(Colormap.LOGARITHM)
    )
    resetButton = dialog._buttonsNonModal.button(qt.QDialogButtonBox.Reset)
    assert resetButton.isEnabled()
    colormap.setEditable(False)
    assert not (resetButton.isEnabled())


def testImageData(qWidgetFactory):
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    data = numpy.random.rand(5, 5)
    dialog.setData(data)


def testEmptyData(qWidgetFactory):
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    data = numpy.empty((10, 0))
    dialog.setData(data)


def testNoneData(qWidgetFactory):
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    data = numpy.random.rand(5, 5)
    dialog.setData(data)
    dialog.setData(None)


def testImageItem(qapp, qWidgetFactory):
    """Check that an ImageData plot item can be used"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormap = Colormap(name="gray", vmin=None, vmax=None)
    data = numpy.arange(3**2).reshape(3, 3)
    item = ImageData()
    item.setData(data, copy=False)

    dialog.setColormap(colormap)
    qapp.processEvents()

    dialog.setItem(item)
    vrange = dialog._getFiniteColormapRange()
    assert vrange == (0, 8)


def testItemDel(qapp, qWidgetFactory):
    """Check that the plot items are not hard linked to the dialog"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormap = Colormap(name="gray", vmin=None, vmax=None)
    data = numpy.arange(3**2).reshape(3, 3)
    item = ImageData()
    item.setData(data, copy=False)

    dialog.setColormap(colormap)
    dialog.show()
    qapp.processEvents()
    dialog.setItem(item)
    previousRange = dialog._getFiniteColormapRange()
    del item
    vrange = dialog._getFiniteColormapRange()
    assert vrange != previousRange


def testDataDel(qapp, qWidgetFactory):
    """Check that the data are not hard linked to the dialog"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    colormap = Colormap(name="gray", vmin=None, vmax=None)
    data = numpy.arange(5)

    dialog.setColormap(colormap)
    qapp.processEvents()

    dialog.setData(data)
    previousRange = dialog._getFiniteColormapRange()
    del data
    vrange = dialog._getFiniteColormapRange()
    assert vrange != previousRange


def testDeleteWhileExec(qWidgetFactory):
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)
    qt.QTimer.singleShot(1000, dialog.deleteLater)
    result = dialog.exec()
    assert result == 0


def testUpdateImageData(qapp, qWidgetFactory):
    """Test that range/histogram takes into account item updates"""
    dialog = qWidgetFactory(ColormapDialog.ColormapDialog)

    item = ImageData()
    item.setColormap(Colormap())
    dialog.setItem(item)
    dialog.setColormap(item.getColormap())
    qapp.processEvents()

    assert dialog._histoWidget.getFiniteRange() == (0, 1)

    item.setData([(1, 2), (3, 4)])

    assert dialog._histoWidget.getFiniteRange() == (1, 4)



# /*##########################################################################
# Copyright (C) 2017-2023 European Synchrotron Radiation Facility
#
# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
# the ESRF by the Software group.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#############################################################################*/
"""Some widget construction to check if a sample moved"""

__author__ = ["H. Payno"]
__license__ = "MIT"
__date__ = "19/03/2018"

import os
import functools
import logging
from silx.gui import qt
from silx.gui import utils as qtutils
from silx.gui.widgets.TableWidget import TableWidget
from silx.io.url import DataUrl, slice_sequence_to_string
from silx.utils.deprecation import deprecated, deprecated_warning
from silx.gui import constants

logger = logging.getLogger(__name__)


class _IntegratedRadioButton(qt.QWidget):
    """RadioButton integrated in the QTableWidget as a centered widget"""

    toggled = qt.Signal()

    def __init__(self, parent=None):
        qt.QWidget.__init__(self, parent=parent)
        self.setContentsMargins(1, 1, 1, 1)
        layout = qt.QHBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(1)

        self._radio = qt.QRadioButton(parent=self)
        self._radio.setObjectName("radio")
        self._radio.setAutoExclusive(False)
        self._radio.setMinimumSize(self._radio.minimumSizeHint())
        self._radio.setMaximumSize(self._radio.minimumSizeHint())
        self._radio.toggled.connect(self.toggled.emit)
        layout.addWidget(self._radio)
        self.setSizePolicy(qt.QSizePolicy.Fixed, qt.QSizePolicy.Fixed)

    def setChecked(self, checked: bool):
        self._radio.setChecked(checked)

    def isChecked(self) -> bool:
        return self._radio.isChecked()


class _DataUrlItem(qt.QTableWidgetItem):
    FILENAME = 0
    DATAPATH = 1
    SLICE = 2

    def __init__(self, url, display: int):
        qt.QTableWidgetItem.__init__(self)
        self._url = url
        self._display = display

        if self._display == self.FILENAME:
            text = os.path.basename(self._url.file_path())
        elif self._display == self.DATAPATH:
            text = self._url.data_path()
        elif self._display == self.SLICE:
            s = self._url.data_slice()
            if s is not None:
                text = slice_sequence_to_string(self._url.data_slice())
            else:
                text = ""
        else:
            raise RuntimeError(f"Unsupported display node: {self._display}")

        toolTip = self._url.path()

        self.setText(text)
        self.setToolTip(toolTip)

    def dataUrl(self):
        return self._url


class UrlSelectionTable(TableWidget):
    """Table used to select the color channel to be displayed for each"""

    FILENAME_COLUMN = 0
    DATAPATH_COLUMN = 1
    SLICE_COLUMN = 2
    IMG_A_COLUMN = 3
    IMG_B_COLUMN = 4
    NB_COLUMNS = 5

    sigImageAChanged = qt.Signal(str)
    """Signal emitted when the image A change. Param is the image url path"""

    sigImageBChanged = qt.Signal(str)
    """Signal emitted when the image B change. Param is the image url path"""

    def __init__(self, parent=None):
        TableWidget.__init__(self, parent)
        self.clear()

    def clear(self):
        qt.QTableWidget.clear(self)
        self.setRowCount(0)
        self.setColumnCount(self.NB_COLUMNS)
        self.setSelectionBehavior(qt.QAbstractItemView.SelectRows)
        self.setSelectionMode(qt.QAbstractItemView.NoSelection)

        item = qt.QTableWidgetItem()
        item.setText("Filename")
        item.setToolTip("Filename to the data")
        self.setHorizontalHeaderItem(self.FILENAME_COLUMN, item)
        item = qt.QTableWidgetItem()
        item.setText("Datapath")
        item.setToolTip("Data path to the dataset")
        self.setHorizontalHeaderItem(self.DATAPATH_COLUMN, item)
        item = qt.QTableWidgetItem()
        item.setText("Slice")
        item.setToolTip("Slice applied to the dataset")
        self.setHorizontalHeaderItem(self.SLICE_COLUMN, item)
        item = qt.QTableWidgetItem()
        item.setText("A")
        item.setToolTip("Selected image as A")
        self.setHorizontalHeaderItem(self.IMG_A_COLUMN, item)
        item = qt.QTableWidgetItem()
        item.setText("B")
        item.setToolTip("Selected image as B")
        self.setHorizontalHeaderItem(self.IMG_B_COLUMN, item)

        self.verticalHeader().hide()
        setSectionResizeMode = self.horizontalHeader().setSectionResizeMode
        setSectionResizeMode(self.FILENAME_COLUMN, qt.QHeaderView.ResizeToContents)
        setSectionResizeMode(self.DATAPATH_COLUMN, qt.QHeaderView.Stretch)
        setSectionResizeMode(self.SLICE_COLUMN, qt.QHeaderView.ResizeToContents)
        setSectionResizeMode(self.IMG_A_COLUMN, qt.QHeaderView.ResizeToContents)
        setSectionResizeMode(self.IMG_B_COLUMN, qt.QHeaderView.ResizeToContents)
        self.setSortingEnabled(True)
        self._checkBoxes = {}

    def setUrls(self, urls: list) -> None:
        """

        :param urls: urls to be displayed
        """
        for url in urls:
            self.addUrl(url=url)

    def addUrl(self, url: DataUrl, **kwargs):
        """
        Append this DataUrl to the end of the list of URLs.

        :param url:
        :param args:
        :return: index of the created items row
        :rtype int
        """
        assert isinstance(url, DataUrl)
        row = self.rowCount()
        self.setRowCount(row + 1)

        item = _DataUrlItem(url, _DataUrlItem.FILENAME)
        item.setFlags(qt.Qt.ItemIsEnabled | qt.Qt.ItemIsSelectable)
        self.setItem(row, self.FILENAME_COLUMN, item)

        item = _DataUrlItem(url, _DataUrlItem.DATAPATH)
        item.setFlags(qt.Qt.ItemIsEnabled | qt.Qt.ItemIsSelectable)
        self.setItem(row, self.DATAPATH_COLUMN, item)

        item = _DataUrlItem(url, _DataUrlItem.SLICE)
        item.setFlags(qt.Qt.ItemIsEnabled | qt.Qt.ItemIsSelectable)
        self.setItem(row, self.SLICE_COLUMN, item)

        widgetImgA = _IntegratedRadioButton(parent=self)
        self.setCellWidget(row, self.IMG_A_COLUMN, widgetImgA)
        callbackImgA = functools.partial(self._activeImgAChanged, row)
        widgetImgA.toggled.connect(callbackImgA)

        widgetImgB = _IntegratedRadioButton(parent=self)
        self.setCellWidget(row, self.IMG_B_COLUMN, widgetImgB)
        callbackImgB = functools.partial(self._activeImgBChanged, row)
        widgetImgB.toggled.connect(callbackImgB)

        self._checkBoxes[row] = {
            self.IMG_A_COLUMN: widgetImgA,
            self.IMG_B_COLUMN: widgetImgB,
        }
        self.resizeColumnsToContents()
        return row

    def _getItemFromUrlPath(self, urlPath: str) -> _DataUrlItem:
        """Returns the Qt item storing this urlPath, else None"""
        for r in range(self.rowCount()):
            item = self.item(r, self.FILENAME_COLUMN)
            url = item.dataUrl()
            if url.path() == urlPath:
                return item
        return None

    def setError(self, urlPath: str, message: str):
        """Flag this urlPath with an error in the UI."""
        item = self._getItemFromUrlPath(urlPath)
        if item is None:
            return
        if message == "":
            item.setIcon(qt.QIcon())
            item.setToolTip("")
        else:
            style = qt.QApplication.style()
            icon = style.standardIcon(qt.QStyle.SP_MessageBoxCritical)
            item.setIcon(icon)
            item.setToolTip(f"Error: {message}")

    def _activeImgAChanged(self, row):
        if self._checkBoxes[row][self.IMG_A_COLUMN].isChecked():
            self._updateCheckBoxes(self.IMG_A_COLUMN, row)
            url = self.item(row, self.FILENAME_COLUMN).dataUrl()
            self.sigImageAChanged.emit(url.path())
        else:
            self.sigImageAChanged.emit(None)

    def _activeImgBChanged(self, row):
        if self._checkBoxes[row][self.IMG_B_COLUMN].isChecked():
            self._updateCheckBoxes(self.IMG_B_COLUMN, row)
            url = self.item(row, self.FILENAME_COLUMN).dataUrl()
            self.sigImageBChanged.emit(url.path())
        else:
            self.sigImageBChanged.emit(None)

    def _updateCheckBoxes(self, column, row):
        for r in range(self.rowCount()):
            if r == row:
                continue
            c = self._checkBoxes[r][column]
            with qtutils.blockSignals(c):
                c.setChecked(False)

    @deprecated(
        replacement="getUrlSelection",
        since_version="2.0",
        reason="Conflict with Qt API",
    )
    def getSelection(self):
        return self.getUrlSelection()

    def setSelection(self, url_img_a, url_img_b):
        if isinstance(url_img_a, qt.QRect):
            return super().setSelection(url_img_a, url_img_b)
        deprecated_warning(
            "Function",
            "setSelection",
            replacement="setUrlSelection",
            since_version="2.0",
            reason="Conflict with Qt API",
        )
        return self.setUrlSelection(url_img_a, url_img_b)

    def getUrlSelection(self):
        """

        :return: url selected for img A and img B.
        """
        imgA = imgB = None
        for row in range(self.rowCount()):
            url = self.item(row, self.FILENAME_COLUMN).dataUrl()
            if self._checkBoxes[row][self.IMG_A_COLUMN].isChecked():
                imgA = url
            if self._checkBoxes[row][self.IMG_B_COLUMN].isChecked():
                imgB = url
        return imgA, imgB

    def setUrlSelection(self, url_img_a, url_img_b):
        """

        :param ddict: key: image url, values: list of active channels
        """
        rowA = None
        rowB = None
        for row in range(self.rowCount()):
            for img in (self.IMG_A_COLUMN, self.IMG_B_COLUMN):
                c = self._checkBoxes[row][img]
                with qtutils.blockSignals(c):
                    c.setChecked(False)
            url = self.item(row, self.FILENAME_COLUMN).dataUrl()
            if url.path() == url_img_a:
                rowA = row
            if url.path() == url_img_b:
                rowB = row

        if rowA is not None:
            c = self._checkBoxes[rowA][self.IMG_A_COLUMN]
            with qtutils.blockSignals(c):
                c.setChecked(True)

        if rowB is not None:
            c = self._checkBoxes[rowB][self.IMG_B_COLUMN]
            with qtutils.blockSignals(c):
                c.setChecked(True)

        self.sigImageAChanged.emit(url_img_a)
        self.sigImageBChanged.emit(url_img_b)

    def removeUrl(self, url):
        raise NotImplementedError("")

    def supportedDropActions(self):
        """Inherited method to redefine supported drop actions."""
        return qt.Qt.CopyAction | qt.Qt.MoveAction

    def mimeTypes(self):
        """Inherited method to redefine draggable mime types."""
        return [constants.SILX_URI_MIMETYPE]

    def dropMimeData(
        self, row: int, column: int, mimedata: qt.QMimeType, action: qt.Qt.DropAction
    ):
        """Inherited method to handle a drop operation to this model."""
        if action == qt.Qt.IgnoreAction:
            return True
        if mimedata.hasFormat(constants.SILX_URI_MIMETYPE):
            urlText = str(mimedata.data(constants.SILX_URI_MIMETYPE), "utf-8")
            url = DataUrl(urlText)
            self.addUrl(url)
            return True
        return False



# /*##########################################################################
#
# Copyright (c) 2023 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""This module provides an abstract PlotToolButton that can be use to create
plot tools for a toolbar.
"""

from __future__ import annotations

__authors__ = ["V. Valls"]
__license__ = "MIT"
__date__ = "20/12/2023"


import logging
import weakref

from silx.gui import qt


_logger = logging.getLogger(__name__)


class PlotToolButton(qt.QToolButton):
    """A QToolButton connected to a :class:`~silx.gui.plot.PlotWidget`."""

    def __init__(self, parent: qt.QWidget | None = None, plot=None):
        super(PlotToolButton, self).__init__(parent)
        self._plotRef = None
        if plot is not None:
            self.setPlot(plot)

    def plot(self):
        """
        Returns the plot connected to the widget.
        """
        return None if self._plotRef is None else self._plotRef()

    def setPlot(self, plot):
        """
        Set the plot connected to the widget

        :param plot: :class:`.PlotWidget` instance on which to operate.
        """
        previousPlot = self.plot()

        if previousPlot is plot:
            return
        if previousPlot is not None:
            self._disconnectPlot(previousPlot)

        if plot is None:
            self._plotRef = None
        else:
            self._plotRef = weakref.ref(plot)
            self._connectPlot(plot)

    def _connectPlot(self, plot):
        """
        Called when the plot is connected to the widget

        :param plot: :class:`.PlotWidget` instance
        """
        pass

    def _disconnectPlot(self, plot):
        """
        Called when the plot is disconnected from the widget

        :param plot: :class:`.PlotWidget` instance
        """
        pass



# /*##########################################################################
#
# Copyright (c) 2004-2016 European Synchrotron Radiation Facility
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# ###########################################################################*/
"""Functions to prepare events to be sent to Plot callback."""

__author__ = ["V.A. Sole", "T. Vincent"]
__license__ = "MIT"
__date__ = "18/02/2016"


import numpy as np


def prepareDrawingSignal(event, type_, points, parameters=None):
    """See Plot documentation for content of events"""
    assert event in ("drawingProgress", "drawingFinished")

    if parameters is None:
        parameters = {}

    eventDict = {}
    eventDict["event"] = event
    eventDict["type"] = type_
    points = np.array(points, dtype=np.float32)
    points.shape = -1, 2
    eventDict["points"] = points
    eventDict["xdata"] = points[:, 0]
    eventDict["ydata"] = points[:, 1]
    if type_ in ("rectangle",):
        eventDict["x"] = eventDict["xdata"].min()
        eventDict["y"] = eventDict["ydata"].min()
        eventDict["width"] = eventDict["xdata"].max() - eventDict["x"]
        eventDict["height"] = eventDict["ydata"].max() - eventDict["y"]
    eventDict["parameters"] = parameters.copy()
    return eventDict


def prepareMouseSignal(eventType, button, xData, yData, xPixel, yPixel):
    """See Plot documentation for content of events"""
    assert eventType in ("mouseMoved", "mouseClicked", "mouseDoubleClicked")
    assert button in (None, "left", "middle", "right")

    return {
        "event": eventType,
        "x": xData,
        "y": yData,
        "xpixel": xPixel,
        "ypixel": yPixel,
        "button": button,
    }


def prepareHoverSignal(label, type_, posData, posPixel, draggable, selectable):
    """See Plot documentation for content of events"""
    return {
        "event": "hover",
        "label": label,
        "type": type_,
        "x": posData[0],
        "y": posData[1],
        "xpixel": posPixel[0],
        "ypixel": posPixel[1],
        "draggable": draggable,
        "selectable": selectable,
    }


def prepareMarkerSignal(
    eventType,
    button,
    label,
    type_,
    draggable,
    selectable,
    posDataMarker,
    posPixelCursor=None,
    posDataCursor=None,
):
    """See Plot documentation for content of events"""
    if eventType == "markerClicked":
        assert posPixelCursor is not None
        assert posDataCursor is None

        posDataCursor = list(posDataMarker)
        if hasattr(posDataCursor[0], "__len__"):
            posDataCursor[0] = posDataCursor[0][-1]
        if hasattr(posDataCursor[1], "__len__"):
            posDataCursor[1] = posDataCursor[1][-1]

    elif eventType == "markerMoving":
        assert posPixelCursor is not None
        assert posDataCursor is not None

    elif eventType == "markerMoved":
        assert posPixelCursor is None
        assert posDataCursor is None

        posDataCursor = posDataMarker
    else:
        raise NotImplementedError("Unknown event type {0}".format(eventType))

    eventDict = {
        "event": eventType,
        "button": button,
        "label": label,
        "type": type_,
        "x": posDataCursor[0],
        "y": posDataCursor[1],
        "xdata": posDataMarker[0],
        "ydata": posDataMarker[1],
        "draggable": draggable,
        "selectable": selectable,
    }

    if eventType in ("markerMoving", "markerClicked"):
        eventDict["xpixel"] = posPixelCursor[0]
        eventDict["ypixel"] = posPixelCursor[1]

    return eventDict


def prepareImageSignal(button, item, col, row, x, y, xPixel, yPixel):
    """See Plot documentation for content of events"""
    return {
        "event": "imageClicked",
        "button": button,
        "item": item,
        "label": item.getName(),
        "type": "image",
        "col": col,
        "row": row,
        "x": x,
        "y": y,
        "xpixel": xPixel,
        "ypixel": yPixel,
    }


def prepareCurveSignal(button, item, xData, yData, x, y, xPixel, yPixel):
    """See Plot documentation for content of events"""
    return {
        "event": "curveClicked",
        "button": button,
        "item": item,
        "label": item.getName(),
        "type": "curve",
        "xdata": xData,
        "ydata": yData,
        "x": x,
        "y": y,
        "xpixel": xPixel,
        "ypixel": yPixel,
    }


def prepareLimitsChangedSignal(sourceObj, xRange, yRange, y2Range):
    """See Plot documentation for content of events"""
    return {
        "event": "limitsChanged",
        "source": id(sourceObj),
        "xdata": xRange,
        "ydata": yRange,
        "y2data": y2Range,
    }
